function [Da Dv nn timeArray] =  AVMP(audio, video, P)
% QINGJU LIU, University of Surrey
%%%%
%%%% obtain all the parameter for coding and learning
P = parameterCoding(audio, video, P);
P = parameterLearning(audio, video, P);
stationary = 0;

usedAtomIndexOld = [];nnOld = [];
usedAtomIndex = [];nn = [];
DaOld = [];DvOld = [];
for iter = 1:P.iterMax,
    %%%%%%%%%
    %     tic
    fprintf('==================================================\n This is the %d -th iteration \n', iter);
    P = updatePvsIteration(P, iter);
    
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % The coding process
    tic
    [ParamCoef Ra Rv Params] = coding(audio, video, P);
    toc
    timeArray(iter) = toc;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % The learning process
    [Da Dv ParamCoef] = learning(ParamCoef, Ra, Rv, P);

    
    P = updateP(DaOld,Da,DvOld,Dv,P,Params);  % P.Da, P.Dv, P.projectionIndex P.similarityIndex are updated
    if mod(iter,5)==0,
        P.similarityIndex = zeros(2,P.atomNum);
    end
    
    %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % stationary mode
    usedAtomIndexOldOld = usedAtomIndexOld; nnOldOld = nnOld;
    usedAtomIndexOld = usedAtomIndex; nnOld = nn;
    atomIndex = ParamCoef.atomIndex;
    usedAtomIndex = unique(atomIndex);
    nn = histc(atomIndex,usedAtomIndex);    %frequency
    
    AtomNum = length(usedAtomIndex);
    if AtomNum<P.atomMinNum, % not enought atoms converged:(
        stationary = 0;
    else
        if iter>=P.iterStop,
            stationary = detectStationary(usedAtomIndex,nn,usedAtomIndexOld,nnOld,usedAtomIndexOldOld,nnOldOld,stationary,P.dispMode);
        end
    end
    
    DaOld = Da;DvOld = Dv;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% save and display
    saveAndDisplay(P,ParamCoef,Da,Dv,timeArray,iter,stationary);
    
    if stationary>=2,  % two successive stationary.
        break
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the finally converged AV dictionary atoms
% usedAtomIndex = unique(atomIndex);
% nn = histc(atomIndex,usedAtomIndex);%frequency
[nn,I] = sort(nn,'descend');
usedAtomIndex = usedAtomIndex(I);
switch P.audioFlag
    case 1
        Da = Da(:,usedAtomIndex);
    case 2
        Da = Da(:,:,usedAtomIndex);
end
Dv = Dv(:,:,:,usedAtomIndex);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function P = updateP(DaOld,Da,DvOld,Dv,P,Params)
Nk = P.atomNum;
P.Da = Da; P.Dv = Dv;
[projectionIndex similarityIndex] = detectProjectionIndex(DaOld,Da,DvOld,Dv,P); %similar the the function detectStationary
P.projectionIndex = projectionIndex;
P.similarityIndex = similarityIndex;
P.paramsOld = Params;

if any(ismember([2], P.similarityArray(2))), %%% using SSIM for visual similarity calculation
    frameStart = P.frameStartEnd(1);
    frameEnd = P.frameStartEnd(2);
    Dv = Dv(:,:,frameStart:frameEnd,:);
    Dv = reshape(Dv,[],Nk);
    uDv = mean(Dv);
    sigmaDv_sqr = mean(Dv.^2)-uDv.^2;
    P.uDv = uDv;
    P.sigmaDv_sqr = sigmaDv_sqr;
end
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function P = updatePvsIteration(P,iter)

changeThresholdIndex = find(ismember(P.thresholdBench, iter));
if any(changeThresholdIndex),
    if isempty(P.realThresholdCell),
        changeThresholdIndex = min(changeThresholdIndex,length(P.thresholdCell));
        P.threshold = P.thresholdCell{changeThresholdIndex};
        P.realThreshold = [];
    else
        changeThresholdIndex = min(changeThresholdIndex,length(P.realThresholdCell));
        P.realThreshold = P.realThresholdCell{changeThresholdIndex};
    end
end


changeNoOverlapIndex = find(ismember(P.noOverlapBench, iter));
if any(changeNoOverlapIndex),
    changeNoOverlapIndex = min(changeNoOverlapIndex,length(P.noOverlapCell));
    P.noOverlap = P.noOverlapCell{changeNoOverlapIndex};
end


if iter>=P.changeAtomBench,
    P.atomMinNumUse = 0; % no longer change atoms
else
    P.atomMinNumUse = P.atomMinNum;
end
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function saveAndDisplay(P,ParamCoef,Da,Dv,timeArray,iter,stationary)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% display text mode
if P.dispMode,
    fprintf(['atomIndex  \t', num2str((1:P.atomNum),'%-5.5g'),'\n']);
    fprintf(['Ca MP      \t', num2str(P.projectionIndex(1,:),'%-5.5g'),'\n']);
    fprintf(['Cv MP      \t', num2str(P.projectionIndex(2,:),'%-5.5g'),'\n']);
    fprintf(['Ca simi    \t', num2str(P.similarityIndex(1,:),'%-5.5g'),'\n']);
    fprintf(['Cv simi    \t', num2str(P.similarityIndex(2,:),'%-5.5g'),'\n']);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% save mode
if (iter==1 || mod(iter,P.saveIter)==0 || stationary>=2) && P.saveMode,
    if isempty(P.seedNumber),
        saveName = ['Dictionary_',num2str(iter,'%03d'),'.mat'];
    else
        saveName = ['Dictionary_',num2str(iter,'%03d'),'_',num2str(P.seedNumber,'%d'),'.mat'];
    end
    save(saveName, 'Da', 'Dv', 'ParamCoef', 'timeArray');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% display figure mode
if (mod(iter,P.figureIter)==0 || iter==1) && P.figureMode,
    showResult(ParamCoef,Da,Dv,P.ExistingFigN,[],[],[],[],[],[],[],10);
end
end
