function [DaShow,DvShow,DaConverge,DvConverge] = ConvergeAtomFromResults(ParamCoef,Da,Dv,maxNum,timeFlag)
% ParamCoef,Da,Dv are saved results in iterations of bootstrap coding
global originalAudio
global originalVideo;

if nargout>2,
    [DaConverge,DvConverge] = ConvergeAtomFromResults1(ParamCoef,Da,Dv);
end

DaShow = [];
DvShow = [];

atomIndex = ParamCoef.atomIndex;
usedAtomIndex = unique(atomIndex);
nn = histc(atomIndex,usedAtomIndex);%frequency
% display the most significant AV atoms
[nn,order] = sort(nn,'descend');
usedAtomIndex = usedAtomIndex(order);
daSize = size(Da); daSize = daSize(1:end-1);daLength = prod(daSize);
dvSize = size(Dv); dvSize = dvSize(1:end-1);dvLength = prod(dvSize);


if nargin<5,
    timeFlag = 0;  % do not care if the audio is time domain or not
elseif isempty(timeFlag)
    timeFlag = 0;
end
if nargin<4,
    showAtomNum = length(usedAtomIndex);
elseif isempty(maxNum),
    showAtomNum = length(usedAtomIndex);
else
    showAtomNum = min(maxNum,length(usedAtomIndex));
end


if length(daSize)==2 && timeFlag,
    dar = daSize(2);
    for i = 1:showAtomNum,
        k = usedAtomIndex(i);
        da_f = Da(:,:,k);
        index = find(atomIndex==k);
        n_k = length(index);
        
        corrMax = 0;
        use_ii = 1;
        for ii = 1:min(n_k,20),
            z = ParamCoef.Translation_v(3,index(ii));
            alpha = ParamCoef.Translation_a(index(ii));
            audio_start = round((z-1)*ParamCoef.FsaVsFsv)+alpha;
            temp = originalAudio(:,audio_start+(0:dar-1));
            da_f1 = abs(temp);
            temp = corrcoef(da_f(:),da_f1(:));
            corrTemp = temp(1,2);
            if corrTemp>corrMax,
                corrMax = corrTemp;
                use_ii = ii;
            end
        end
        z = ParamCoef.Translation_v(3,index(use_ii));
        alpha = ParamCoef.Translation_a(index(use_ii));
        audio_start = round((z-1)*ParamCoef.FsaVsFsv)+alpha;
        temp = originalAudio(:,audio_start+(0:dar-1));
        atom_a = istft(temp);
        DaShow(:,i) = atom_a;
    end
else
    Da = reshape(Da,daLength,[]);
    DaShow = zeros(daLength,showAtomNum);
    for i = 1:showAtomNum,
        k = usedAtomIndex(i);
        da = Da(:,k);
        DaShow(:,i) = da;
    end
    DaShow = reshape(DaShow,[daSize showAtomNum]);
end


if ~isempty(originalVideo),
    for i = 1:showAtomNum,
        k = usedAtomIndex(i);
        
        index = find(atomIndex==k);
        n_k = length(index);
        [H,W,Frame] = size(originalVideo);
        frame = dvSize(3);
        tempVideo = zeros(H,W,frame,n_k);
        for ii = 1:n_k,
            z = ParamCoef.Translation_v(3,index(ii));
            tempVideo(:,:,:,ii) = originalVideo(:,:,z+(0:frame-1));
        end
        dv = mean(tempVideo,4);
        DvShow(:,:,:,i) = dv;
    end
    
else
    Dv = reshape(Dv,dvLength,[]);
    DvShow = zeros(dvLength,showAtomNum);
    for i = 1:showAtomNum,
        k = usedAtomIndex(i);
        dv = Dv(:,k);
        DvShow(:,i) = dv;
    end
    DvShow = reshape(DvShow,[dvSize showAtomNum]);
end

