function [scanIndex visualScan audioScan] =  InitialScanIndexLip(audio, video, sizeVideoAtom, frameDynamic, threshold, useAreaFrame)
% Qingju Liu
% input:
%   useAreaFrame:       Pre-defination of the frames being scanned.
%   Mx2 matrix. e.g.    [5 10] or [5 10; 30 40]
%   frameDynamic:      for calculation of dynamic visual features
%   video:             |Fourier transform (video)|, the extracted lip

%
% output
%   scanIndex:      the logic matrix associated with each pixel
%   VisualScan:     the variance of the temporary videoAtom-sized video part
%   audioScan:      the audio 'energy' for weighting


if nargin<3,
    error('At least three inputs are needed')
end

if numel(audio) == length(audio),   % 1 D
    audio = abs(audio);
else                                % 2 D---->1 D
    audio = sum(abs(audio));
end
audio_length = length(audio);
[Vr Vc Frame] = size(video);
dvr = sizeVideoAtom(1);
dvc = sizeVideoAtom(2);
frame = sizeVideoAtom(3);
FsaVsFsv = audio_length/Frame;
dar = round(FsaVsFsv*frame);
ySearch = Vr-dvr+1;
xSearch = Vc-dvc+1;
zSearch = Frame-frame+1;

if nargin<6,
    useAreaFrame = [1 Frame];% use all the frames
end

if nargin>4 && ~isempty(threshold),
    assert(threshold>=0 && threshold<=1, 'Error!')
else
    threshold=0.01;
end
if nargin>3 && ~isempty(frameDynamic),
    fff = frameDynamic;
else
    fff = 10*frame;
end
% % calculate the dynamic visual feature
% dVideo = videoTemporalWhitening(video,0.8,Frame);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% First obtain the audioScan
audioScan = zeros(1,zSearch);
for tBlock = 1:size(useAreaFrame,1)     % for each snipplet
    frameStart = useAreaFrame(tBlock,1);
    frameEnd = useAreaFrame(tBlock,2);
    audioStart = round((frameStart-1)*FsaVsFsv)+1;
    audioEnd = round(frameEnd*FsaVsFsv);
    subAudio = audio(audioStart:audioEnd);
    N = (frameEnd-frameStart-frame+2);
    Index = round((0:N-1)*FsaVsFsv); Index(end) = Index(end)-1;
    Index = repmat(Index,[dar,1])+repmat((1:dar)',[1 N]);
    subAudio = subAudio(Index);
    subAudio = sum(subAudio);
    subAudio(subAudio<0.05*mean(subAudio)) = 0;
    temp = frameEnd-frameStart-frame+1;
    audioScan(frameStart+(0:temp)) = subAudio;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Then obtain the visualScan
M = zeros(1,zSearch);
SS = max(1,ceil(fff*0.25));
for f = 1:SS:Frame-fff+1,
    tempVideof = video(:,:,f:(f+fff-1));
    tempDynamic = var(tempVideof,0,3);
    tempDynamic = squeeze(mean(tempDynamic(:)));
    if isnan(tempDynamic),
        tempDynamic = 0.001;
    end
    if f==1,
        M(1:fff) = tempDynamic;
    else
        M(f+fff-SS:f+fff-1) = tempDynamic;
        M(f:f+fff-SS-1) = min(M(f:f+fff-SS-1),tempDynamic);
    end
end
if (f+fff-1)<Frame,
    M(f+fff:end) = M(f+fff-1);
end
M = M(1:zSearch);
M(M>0.001) = 0.001;

SS = min(SS,round(frame/3));
M_left = [M(SS+1:end),ones(1,SS)*M(end)];
M_right = [ones(1,SS)*M(1), M(1:end-SS)];
M = max([M;M_left;M_right]);
M = reshape(M,[1 1 zSearch]);


visualScan = M;
audioScan(audioScan>0) = 1;
AudioVisualScan = repmat(reshape(audioScan,[1 1 zSearch]),[ySearch xSearch 1]) .* visualScan;

% thresholdValue = mean(AudioVisualScan(:))*threshold;
thresholdValue = 3*10^-4;
scanIndex = AudioVisualScan>thresholdValue;
