function [params, AtomP]= atomProjection(audio,video,AtomP)
% list the parameter definition for each AV atom Projection
projectionIndex = AtomP.projectionIndex;
similarityIndex = AtomP.similarityIndex;



if all(projectionIndex==[0;0]),
    params = AtomP.paramsOld;
else
    %%%  find the parameters in AVMPcodingParameterDefinition.m
    audioFlag = AtomP.audioFlag;
    if ~isempty(AtomP.realThreshold),
        realThresholdValue_a = AtomP.realThreshold(1);
        realThresholdValue_v = AtomP.realThreshold(2);
    else
        threshold_a = AtomP.threshold(1);
        threshold_v = AtomP.threshold(2);
        maxCa = AtomP.maxCa;
        maxCv = AtomP.maxCv;
        realThresholdValue_a = threshold_a*maxCa;
        realThresholdValue_v = threshold_v*maxCv;
    end
    scanIndex = AtomP.scanIndex;
    hopSize = AtomP.hopSize;
    frameStart = AtomP.frameStartEnd(1);
    frameEnd = AtomP.frameStartEnd(2);
    similarityArray = AtomP.similarityArray;
    dispMode = AtomP.dispMode;
    FsaVsFsv = AtomP.FsaVsFsv;
    sizeAudioAtom = AtomP.sizeAudioAtom;
    sizeVideoAtom = AtomP.sizeVideoAtom;
    yxzSearch = AtomP.yxzSearch;
    nint = AtomP.nint;
    u1 = AtomP.u1;
    sigma1_sqr = AtomP.sigma1_sqr;
    u_dv = AtomP.u_dv;
    sigma_dv_sqr = AtomP.sigma_dv_sqr;
    da = AtomP.da;
    dv = AtomP.dv;  dv = dv(:,:,frameStart:frameEnd);
    paramsOld = AtomP.paramsOld;
    affectedCoordinates = AtomP.affectedCoordinates;
    
    if similarityArray(2)==99,  % specific for lip region. 
        videoFirstFlag = true;  % first calculate Cv, then Ca
    else
        dv = dv(:);
        videoFirstFlag = false;
    end
    
    yHop = hopSize(1);
    xHop = hopSize(2);
    zHop = hopSize(3);
    
    ySearch = yxzSearch(1);
    xSearch = yxzSearch(2);
    zSearch = yxzSearch(3);
    
    
    if ~isempty(paramsOld),
        ALPHA = paramsOld.ALPHA;
        Ca = paramsOld.Ca;
        Coefa = paramsOld.Coefa;
        Cv = paramsOld.Cv;
        Cav = paramsOld.Cav;
    else
        ALPHA = zeros(1,zSearch);
        Ca = zeros(1,zSearch);
        Coefa = zeros(1,zSearch);
        Cv = zeros(ySearch,xSearch,zSearch);
        Cav = zeros(ySearch,xSearch,zSearch);
        params = struct( 'ALPHA',ALPHA,'Coefa',Coefa,'Ca',Ca,'Cv',Cv,'Cav',Cav);
    end
    
    dar = AtomP.dar;
    dvr = sizeVideoAtom(1);
    dvc = sizeVideoAtom(2);
    frame = sizeVideoAtom(3);
    ffframe = frameEnd-frameStart+1;
    
    
    affctedYstart = affectedCoordinates(1,1);affctedYend = affectedCoordinates(2,1);
    affctedXstart = affectedCoordinates(1,2);affctedXend = affectedCoordinates(2,2);
    affctedZstart = affectedCoordinates(1,3);affctedZend = affectedCoordinates(2,3);
    
    tempSearch = false(ySearch,xSearch);
    tempSearch(1:yHop:end,1:xHop:end) = true;
    temp_u1 = [];
    temp_sigma1_sqr = [];
    for z = affctedZstart:zHop:affctedZend, % z shift
        tempScanZ = scanIndex(:,:,z);
        if any(tempScanZ(:)),
            if videoFirstFlag, %%% firs calculat Cv, then Ca
                %%
                if projectionIndex(2) == 1,
                    %calculate Cv.
                    tempScan = false(ySearch,xSearch);
                    tempScan(affctedYstart:affctedYend,affctedXstart:affctedXend) = tempScanZ(affctedYstart:affctedYend,affctedXstart:affctedXend);
                    tempScan = tempScan & tempSearch;
                    if similarityIndex(2) == 1, % works only for the cours projection with large grid
                        tempCv = Cv(:,:,z);
                        tempScan = tempScan & (tempCv>realThresholdValue_v*0.5);
                    end
                    
                    if any(tempScan(:)),
                        % extract the video patches at the scanning point in the affected area.
                        idx = find(tempScan);
                        [y,x] = ind2sub(size(tempScan),idx);
                        NN = length(idx);
                        videoTensor = zeros(dvr,dvc,ffframe,NN);
                        for iii = 1:NN,
                            videoTensor(:,:,:,iii) = video(y(iii)+(0:dvr-1),x(iii)+(0:dvc-1),z+frameStart-1:z+frameEnd-1);
                        end
                        videoTensor = reshape(videoTensor,[],NN);
                        
                        if ~isempty(u1),
                            temp_u1 = u1(:,:,z);
                            temp_u1 = temp_u1(tempScan);
                            temp_sigma1_sqr = sigma1_sqr(:,:,z);
                            temp_sigma1_sqr =temp_sigma1_sqr(tempScan);
                        end
                        %Important: it decides which kind of objective function to use:)
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        c_v_temp_array = visualSimilarityCalculation(dv,videoTensor,similarityArray(2),ffframe,temp_u1,temp_sigma1_sqr,u_dv,sigma_dv_sqr);
                        
                        if isempty(AtomP.realThreshold),
                            if any(c_v_temp_array>maxCv),
                                maxCv = max(c_v_temp_array);
                                realThresholdValue_v = threshold_v*maxCv;
                            end
                        end
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        tempCv = Cv(:,:,z);
                        tempCv(tempScan) = c_v_temp_array;
                        Cv(:,:,z) = tempCv;
                    end
                end
                
                tempCv = Cv(:,:,z);
                temp =tempCv(tempScanZ);
                if any(temp>realThresholdValue_v) ,
                    % If all Cv is very small, then there is no need to bother to calculate the associated Cv
                    % audio similarity in the affected temporal positon
                    % get audio atom coefficents,
                    if projectionIndex(1) == 1,
                        tempScan = 1;
                        if similarityIndex(1) == 1,% works only for the cours projection with large grid
                            tempScan = Ca(z)>realThresholdValue_a*0.5;
                        end
                        if tempScan,
                            [Ca(z) ALPHA(z) Coefa(z)] = audioSimilarityCalculation(da,audio,z,FsaVsFsv,dar,nint,similarityArray(1),audioFlag);
                            if isempty(AtomP.realThreshold),
                                if Ca(z)>maxCa,
                                    maxCa = Ca(z);
                                    realThresholdValue_a = threshold_a*Ca(z);
                                end
                            end
                        end
                    end
                    tempCv = Cv(:,:,z);
                    tempCav = Cav(:,:,z);
                    tempCav(tempScanZ) = tempCv(tempScanZ) * Ca(z);
                    Cav(:,:,z) = tempCav;
                end
            else
                %%
                % audio similarity in the affected temporal positon
                % get audio atom coefficents,
                if projectionIndex(1) == 1,
                    tempScan = 1;
                    if similarityIndex(1) == 1,
                        tempScan = Ca(z)>realThresholdValue_a*0.5;
                    end
                    if tempScan,
                        [Ca(z) ALPHA(z) Coefa(z)] = audioSimilarityCalculation(da,audio,z,FsaVsFsv,dar,nint,similarityArray(1),audioFlag);
                        if isempty(AtomP.realThreshold),
                            if Ca(z)>maxCa,
                                maxCa = Ca(z);
                                realThresholdValue_a = threshold_a*Ca(z);
                            end
                        end
                    end
                end
                
                % The only difference is the removal of Cv calculation
                if Ca(z)>realThresholdValue_a, % If Ca is very small, then there is no need to bother to calculate the associated Cv
                    if projectionIndex(2) == 1,
                        
                        %calculate Cv.
                        tempScan = false(ySearch,xSearch);
                        tempScan(affctedYstart:affctedYend,affctedXstart:affctedXend) = ...
                            scanIndex(affctedYstart:affctedYend,affctedXstart:affctedXend,z);
                        tempScan = tempScan & tempSearch;
                        if similarityIndex(2) == 1,
                            tempCv = Cv(:,:,z);
                            tempScan = tempScan & (tempCv>realThresholdValue_v*0.5);
                        end
                        
                        if any(tempScan(:)),
                            % extract the video patches at the scanning point in the affected area.
                            idx = find(tempScan);
                            [y,x] = ind2sub(size(tempScan),idx);
                            NN = length(idx);
                            videoTensor = zeros(dvr,dvc,ffframe,NN);
                            for iii = 1:NN,
                                videoTensor(:,:,:,iii) = video(y(iii)+(0:dvr-1),x(iii)+(0:dvc-1),z+frameStart-1:z+frameEnd-1);
                            end
                            videoTensor = reshape(videoTensor,[],NN);
                            
                            if ~isempty(u1),
                                temp_u1 = u1(:,:,z);
                                temp_u1 = temp_u1(tempScan);
                                temp_sigma1_sqr = sigma1_sqr(:,:,z);
                                temp_sigma1_sqr =temp_sigma1_sqr(tempScan);
                            end
                            %Important: it decides which kind of objective function to use:)
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            c_v_temp_array = visualSimilarityCalculation(dv,videoTensor,similarityArray(2),ffframe,temp_u1,temp_sigma1_sqr,u_dv,sigma_dv_sqr);
                            
                            if isempty(AtomP.realThreshold),
                                if any(c_v_temp_array>maxCv),
                                    maxCv = max(c_v_temp_array);
                                    realThresholdValue_v = threshold_v*maxCv;
                                end
                            end
                            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                            tempCv = Cv(:,:,z);
                            tempCv(tempScan) = c_v_temp_array;
                            Cv(:,:,z) = tempCv;
                        end
                    end
                    tempCv = Cv(:,:,z);
                    tempCav = Cav(:,:,z);
                    tempCav(tempScanZ) = tempCv(tempScanZ) * Ca(z);
                    Cav(:,:,z) = tempCav;
                end
            end
        end
    end
    
    % update the maximum similarity values
    if isempty(AtomP.realThreshold),
        AtomP.maxCa = maxCa;
        AtomP.maxCv = maxCv;
    end
    
    params.ALPHA = ALPHA;
    params.Ca = Ca;
    params.Coefa = Coefa;
    params.Cv = Cv;
    params.Cav = Cav;
end
