function [params, AtomP]= atomProjection_Monaci(audio,video,AtomP)
% list the parameter definition for each AV atom Projection
projectionIndex = AtomP.projectionIndex;



if all(projectionIndex==[0;0]),
    params = AtomP.paramsOld;
else
    %%%  find the parameters in AVMPcodingParameterDefinition.m
    audioFlag = AtomP.audioFlag;
    if ~isempty(AtomP.realThreshold),
        realThresholdValue_a = AtomP.realThreshold(1);
    else
        threshold_a = AtomP.threshold(1);
        maxCa = AtomP.maxCa;
        maxCv = AtomP.maxCv;
        realThresholdValue_a = threshold_a*maxCa;
    end
    scanIndex = AtomP.scanIndex;
    hopSize = AtomP.hopSize;
    frameStart = AtomP.frameStartEnd(1);
    frameEnd = AtomP.frameStartEnd(2);
    FsaVsFsv = AtomP.FsaVsFsv;
    sizeVideoAtom = AtomP.sizeVideoAtom;
    yxzSearch = AtomP.yxzSearch;
    nint = AtomP.nint;
    
    %%%
    da = AtomP.da;
    dv = AtomP.dv;  dv = dv(:,:,frameStart:frameEnd);
    dv = dv(:);
    paramsOld = AtomP.paramsOld;
    affectedCoordinates = AtomP.affectedCoordinates;
    
    yHop = hopSize(1);
    xHop = hopSize(2);
    zHop = hopSize(3);
    
    ySearch = yxzSearch(1);
    xSearch = yxzSearch(2);
    zSearch = yxzSearch(3);
    
    
    if ~isempty(paramsOld),
        ALPHA = paramsOld.ALPHA;
        Ca = paramsOld.Ca;
        Coefa = paramsOld.Coefa;
        Cv = paramsOld.Cv;
        Coefv = paramsOld.Coefv;
        Cav = paramsOld.Cav;
    else
        ALPHA = zeros(1,zSearch);
        Ca = zeros(1,zSearch);
        Coefa = zeros(1,zSearch);
        Cv = zeros(ySearch,xSearch,zSearch);
        Coefv = zeros(ySearch,xSearch,zSearch);
        Cav = zeros(ySearch,xSearch,zSearch);
        params = struct( 'ALPHA',ALPHA,'Coefa',Coefa,'Ca',Ca,'Cv',Cv,'Coefv',Coefv,'Cav',Cav);
    end
    
    dar = AtomP.dar;
    dvr = sizeVideoAtom(1);
    dvc = sizeVideoAtom(2);
    frame = sizeVideoAtom(3);
    ffframe = frameEnd-frameStart+1;
    
    
    affctedYstart = affectedCoordinates(1,1);affctedYend = affectedCoordinates(2,1);
    affctedXstart = affectedCoordinates(1,2);affctedXend = affectedCoordinates(2,2);
    affctedZstart = affectedCoordinates(1,3);affctedZend = affectedCoordinates(2,3);
    
    tempSearch = false(ySearch,xSearch);
    tempSearch(1:yHop:end,1:xHop:end) = true;
    
    
    for z = affctedZstart:zHop:affctedZend, % z shift
        tempScanZ = scanIndex(:,:,z);
        if any(tempScanZ(:)),
            if projectionIndex(1) == 1,
                [Ca(z) ALPHA(z) Coefa(z)] = audioSimilarityCalculation(da,audio,z,FsaVsFsv,dar,nint,1,audioFlag);
                if isempty(AtomP.realThreshold),
                    if Ca(z)>maxCa,
                        maxCa = Ca(z);
                        realThresholdValue_a = threshold_a*Ca(z);
                    end
                end
            end
            
            % The only difference is the removal of Cv calculation
            if Ca(z)>realThresholdValue_a, % If Ca is very small, then there is no need to bother to calculate the associated Cv
                if projectionIndex(2) == 1,
                    %calculate Cv.
                    tempScan = false(ySearch,xSearch);
                    tempScan(affctedYstart:affctedYend,affctedXstart:affctedXend) = ...
                        scanIndex(affctedYstart:affctedYend,affctedXstart:affctedXend,z);
                    tempScan = tempScan & tempSearch;
                    
                    if any(tempScan(:)),
                        % extract the video patches at the scanning point
                        % in the affected area. 
                        idx = find(tempScan);
                        [y,x] = ind2sub(size(tempScan),idx);
                        NN = length(idx);
                        videoTensor = zeros(dvr,dvc,ffframe,NN);
                        for iii = 1:NN,
                            videoTensor(:,:,:,iii) = video(y(iii)+(0:dvr-1),x(iii)+(0:dvc-1),z+frameStart-1:z+frameEnd-1);
                        end
                        videoTensor = reshape(videoTensor,[],NN);

                        %Important: it decides which kind of objective function to use:)
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        Coefv_temp_array = dv'*videoTensor;
                        c_v_temp_array = max(Coefv_temp_array,0);
                        if isempty(AtomP.realThreshold),
                            if any(c_v_temp_array>maxCv),
                                maxCv = max(c_v_temp_array);
                            end
                        end
                        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        tempCoefv = Coefv(:,:,z);
                        tempCoefv(tempScan) = Coefv_temp_array;
                        Coefv(:,:,z) = tempCoefv;
                        
                        tempCv = Cv(:,:,z);
                        tempCv(tempScan) = c_v_temp_array;
                        Cv(:,:,z) = tempCv;
                    end
                end
                tempCv = Cv(:,:,z);
                tempCav = Cav(:,:,z);
                tempCav(tempScanZ) = tempCv(tempScanZ) + Ca(z);
                Cav(:,:,z) = tempCav;
            end
        end
    end
    
    % update the maximum similarity values
    if isempty(AtomP.realThreshold),
        AtomP.maxCa = maxCa;
        AtomP.maxCv = maxCv;
    end
    
    params.ALPHA = ALPHA;
    params.Ca = Ca;
    params.Coefa = Coefa;
    params.Cv = Cv;
    params.Coefv = Coefv;
    params.Cav = Cav;
end
