function [AudioSimilarity alpha Ca] = audioSimilarityCalculation(da,audio,z,FsaVsFsv,dar,nint,similarityType,audioFlag)
% Qingju Liu, University of Surrey
% AudioSimilarity: The objective function that we want to maximize
% alpha: the shift at which the optimal audio segment is choosen for one
% specific frame
% Ca: the inner product of the audio atom with the optimal audio segment

if audioFlag == 1,
    lengthAudio = length(audio);
else
    lengthAudio = size(audio,2);
end


OriginalStart1 = min(round((z-1)*FsaVsFsv)+1,lengthAudio-dar+1);
OriginalStart2 = min(round((z-1)*FsaVsFsv)+nint,lengthAudio-dar+1);

startIndex1 = OriginalStart1;
startIndex2 = OriginalStart2;
if startIndex1==startIndex2, % only one segment
    startChooseArray = startIndex1:startIndex2;
    [AudioSimilarity OptIndex Ca] = CaCalculation(da,audio,startChooseArray,dar,similarityType,audioFlag);
else
    array= startIndex1:startIndex2;
    while(length(array)>1)
        resolution = max(1,round(length(array)/10));
        startChooseArray = startIndex1:resolution:startIndex2;
        [AudioSimilarity OptIndex Ca] = CaCalculation(da,audio,startChooseArray,dar,similarityType,audioFlag);
        startIndex1 = max(OptIndex-resolution+1,OriginalStart1);
        startIndex2 = min(OptIndex+resolution-1,OriginalStart2);
        array= startIndex1:startIndex2;
    end
end
alpha = OptIndex-OriginalStart1+1;

function [AudioSimilarity OptIndex Ca] = CaCalculation(da,audio,startChooseArray,dar,similarityType,audioFlag)

tran_pos = length(startChooseArray);
audioIndex = repmat([0:dar-1]',1,tran_pos)+repmat(startChooseArray,dar,1);
if audioFlag == 1,
    tempAudio = audio(audioIndex);
else
    tempAudio = audio(:,audioIndex);
end
tempAudio = reshape(tempAudio, [], tran_pos);
da = da(:);


temp = da'*tempAudio; % inner products
switch similarityType,
    case 2          % The overall result: inner(A,B) * corr(A,B),
                    % Do not Try this for 1D data, it is EXTREMELY time consuming
        temp2 = zeros(1,tran_pos);
        for i = 1:tran_pos,      % corrcoef
            corr = corrcoef(da,tempAudio(:,i));
            temp2(i) = corr(1,2);
        end
        [AudioSimilarity, alpha] = max(temp.*temp2);
        Ca = temp(alpha);
    case 3           % The overall result: corr(A,B),
        temp2 = zeros(1,tran_pos);
        for i = 1:tran_pos,
            corr = corrcoef(da,tempAudio(:,i));
            temp2(i) = corr(1,2);
        end
        [AudioSimilarity, alpha] = max(temp2);
        Ca = temp(alpha);
    otherwise       % The overall result: |inner products|
        [AudioSimilarity, alpha] = max(abs(temp));
        Ca = temp(alpha);
end
% AudioSimilarity(AudioSimilarity<10^-6) = 10^-6;
AudioSimilarity(AudioSimilarity<0) = 0;
OptIndex = startChooseArray(alpha);

% function [AudioSimilarity alpha Ca] = audioSimilarityCalculation(da,audio,z,FsaVsFsv,dar,nint,similarityType,audioFlag)
% % Qingju Liu, University of Surrey
% % AudioSimilarity: The objective function that we want to maximize
% % alpha: the shift at which the optimal audio segment is choosen for one
% % specific frame
% % Ca: the inner product of the audio atom with the optimal audio segment
%
% audio_start = round((z-1)*FsaVsFsv)+1;
% audio_end = min(audio_start+dar+nint-2,length(audio));
%
% tran_pos = audio_end-audio_start-dar+2;
% tempIndex = repmat([1:dar]',1,tran_pos)+repmat([0:tran_pos-1],dar,1);
% if audioFlag == 1,
%     %%%%%%
%     %%%%The first situation, if the audio and da are 1D signal
%     tempAudio = audio(audio_start:audio_end);
%     tempAudio = tempAudio(tempIndex);
% else
%     %%%%%%
%     %%%%The second situation, if the audio and da are 2D signal (PSD)
%     tempAudio = audio(:,audio_start:audio_end);
%     tempAudio = tempAudio(:,tempIndex);
% end
% tempAudio = reshape(tempAudio, [], tran_pos);
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% switch similarityType,
%     case 2          % suppose A = da; B = one column of tempAudio,
%                     % then the overall result: |inner(A,B)| * |corr(A,B)|,
%                     % which is equivalent to |inner(A,B)|^2/norm(B) suppose
%                     % both A and B is zero-mean.
%                     % Do not Try this for 1D data, it is EXTREMELY time consuming
%         temp = da(:)'*tempAudio;
%         temp2 = (temp.^2)./(sqrt(sum(tempAudio.^2)));
%         [AudioSimilarity, alpha] = max(temp.*temp2);
%         Ca = temp(alpha);
%     otherwise       % inner products
%         temp = da(:)'*tempAudio;
%         [AudioSimilarity, alpha] = max(abs(temp));
%         Ca = temp(alpha);
% end
%
% % temptemp = da(:)'*tempAudio; % inner products
% % [tempSimilarity, alpha] = max(abs(temptemp);
% % switch similarityType,
% %     case 1
% %         AudioSimilarity = tempSimilarity;
% %     case 2
% %         temp = corrcoef(da(:),tempAudio(:,alpha));
% %         temp = max(temp(1,2),0);
% %         AudioSimilarity = tempSimilarity*temp;
% %     case 3
% %         temp = corrcoef(da(:),tempAudio(:,alpha));
% %         AudioSimilarity = max(temp(1,2),0);
% %     otherwise %default audio similarity calculation
% %         AudioSimilarity = tempSimilarity;
% % end
% % Ca = temptemp(alpha);
% end