function [DaNew DvNew] = changeAtoms(ParamCoef,Ra,Rv,P)
% Qingju Liu, University of Surrey
audioFlag = P.audioFlag;
thresholdNum = P.thresholdNum;
atomMinNumUse = P.atomMinNumUse;
varThreshold = P.varThreshold;
similarityArray = P.similarityArray;
scanIndex = P.scanIndex;
dispMode = P.dispMode;
FsaVsFsv = P.FsaVsFsv;
frameStart = P.frameStartEnd(1);
frameEnd = P.frameStartEnd(2);

Da = P.Da;
Dv = P.Dv;
DvNew = Dv;
DaNew = Da;


%% important here
Dv = Dv(:,:,frameStart:frameEnd,:);


[dvr dvc frame Nk] = size(Dv);



% Nk is the total number of atoms e.g. 6
% just update 2 of the unused atoms

atomIndex = ParamCoef.atomIndex;% used atom index eg. 2 2 5 3 3 3 3
usedAtomIndex = unique(atomIndex); %e.g. 2 3 5
Num_usedAtomIndex = histc(atomIndex,usedAtomIndex); %e.g. 2 4 1
unusedAtomIndex = setdiff([1:Nk],usedAtomIndex); %e.g. 1 4 6

% update the unused/seldom_used atoms only with the following conditions.

%++++++++++++++++++++++++++++++++++++++++CONDITION 1
% In the previous coding iteration, there are at least one atom that are not
% used or used just for too few times (Please find the varThreshold in
% DeleteAtomsTooFew.mat) E.g. dictionary size is 10, and use 1000 atoms to
% represent a signal, the 2ed atom d2 are used only 4 times, then we
% consider the second atom as not used at all.
ChangeFlag = false;
ChangeFlag2 = false;
mustChangeFlag = false;
ChangeFlagMore = 0;
CreateFlagMore = 0; % generate new Dictionary atoms istead of just spliting one atom.
if length(unusedAtomIndex)>1,% at least two atoms are not used
    ChangeFlag = true;
    ChangeFlag2 = true;
    unusedAtomIndex = unusedAtomIndex(randperm(length(unusedAtomIndex)));
elseif length(unusedAtomIndex)==1, % just one atom is not used
    ChangeFlag = true;
end

if ~ChangeFlag,
    if dispMode,
        disp('All atoms are used and the atom distribution is even, therefore no atom-change.');
    end
    return
end

if atomMinNumUse>length(usedAtomIndex),
    mustChangeFlag = true;
    if atomMinNumUse-length(usedAtomIndex)>8 && max(Num_usedAtomIndex)>40,
        ChangeFlagMore = 5;
    elseif atomMinNumUse-length(usedAtomIndex)>5 && max(Num_usedAtomIndex)>20,
        ChangeFlagMore = 3;
    elseif atomMinNumUse-length(usedAtomIndex)>4 && max(Num_usedAtomIndex)>10,
        ChangeFlagMore = 2;
    elseif atomMinNumUse-length(usedAtomIndex)>5
        CreateFlagMore = 3;
    end
end


%++++++++++++++++++++++++++++++++++++++++CONDITION 2
% E.g. if d2 is located 20 times to represent a signal, and the associated
% 20 video parts have a huge variance, then 20 video parts are grouped to
% two groups via kmeans to update to new atoms. The updated atom index
% is in the vector unusedAtomIndex


if mustChangeFlag
    videoVar = -1;
else
    % first calculate the gross variance of all the allocated visual atoms
    totalNum = length(atomIndex);
    permuteAtomIndex = randperm(totalNum);
    totalNumUse = min(50,totalNum);
    permuteAtomIndex = permuteAtomIndex(1:totalNumUse);
    y_array = ParamCoef.Translation_v(1,permuteAtomIndex);
    x_array = ParamCoef.Translation_v(2,permuteAtomIndex);
    z_array = ParamCoef.Translation_v(3,permuteAtomIndex);
    
    videoTensor = zeros(dvr,dvc,frame,totalNumUse);
    for i = 1:totalNumUse
        videoTensor(:,:,:,i) = Rv(y_array(i)+(0:dvr-1),x_array(i)+(0:dvc-1),z_array(i)+(frameStart-1:frameEnd-1));
    end
    videoTensor = reshape(videoTensor,[],totalNumUse);
    
    videoVar = abs(videoTensor - repmat(mean(videoTensor,2),[1 totalNumUse]));
    videoVar = mean(videoVar(:)) + eps;
end


videoVarMax = 0;
ChangeFlag = false;
for kk = randperm(length(usedAtomIndex)),
    k = usedAtomIndex(kk);              % the k-th atom
    n_k = Num_usedAtomIndex(kk);        % frquency of the k-th atom
    index = atomIndex==k;               % when/where the k-th atom occured
    if n_k>=4,
        y_array = ParamCoef.Translation_v(1,index);
        x_array = ParamCoef.Translation_v(2,index);
        z_array = ParamCoef.Translation_v(3,index);
        videoTensor = zeros(dvr,dvc,frame,n_k);
        for i = 1:n_k
            videoTensor(:,:,:,i) = Rv(y_array(i)+(0:dvr-1),x_array(i)+(0:dvc-1),z_array(i)+(frameStart-1:frameEnd-1));
        end
        videoTensor = reshape(videoTensor,[],n_k);
        xVar = abs(videoTensor - repmat(mean(videoTensor,2),[1 n_k]));
        xVar= mean(xVar(:)) + eps;
        
        if xVar>varThreshold*videoVar,
            ChangeFlag = true;
            if xVar>videoVarMax,
                % we get one atom that can be divided via K-means to obtain
                % two atoms and change unused ones.
                videoVarMax = xVar; update_k = k; update_n_k = n_k;
            end
        end
    end
end


if ~ChangeFlag,
    if dispMode,
        disp('Atom variance is small, therefore no atom-change.');
    end
    return
end

%++++++++++++++++++++++++++++++++++++++++Now change Atom

%the k-th atoms are used  and with the largest variance.
index = atomIndex==update_k;
% concatnate update_n_k video signals where the k-th atoms occured
y_array = ParamCoef.Translation_v(1,index);
x_array = ParamCoef.Translation_v(2,index);
z_array = ParamCoef.Translation_v(3,index);

X = zeros(dvr,dvc,frame,update_n_k);
for i = 1:update_n_k
    X(:,:,:,i) = Rv(y_array(i)+(0:dvr-1),x_array(i)+(0:dvc-1),z_array(i)+(frameStart-1:frameEnd-1));
end
X = reshape(X,[],update_n_k);
X = X';


% k-means are used to cluster the k-th atoms into two parts and
% then update the atoms

try
    [IDX, C] = kmeans(X, 2);
catch
    update_n_k = floor(update_n_k/2)*2;
    X = X(1:update_n_k,:);
    if update_n_k==2,
        C = X;
    else
        X = reshape(X,[],2,size(X,2));
        C = squeeze(mean(X));
    end
end


% change one atom
DvNew(:,:,frameStart:frameEnd,unusedAtomIndex(1)) = reshape(C(1,:),[dvr dvc frame]);
if P.similarityArray(2)==99,
    for frameIndex = frameStart:frameEnd,
        tempdv = DvNew(:,:,frameIndex,unusedAtomIndex(1));
        inner_c = tempdv(23:38,:);
        DvNew(39,1,frameIndex,unusedAtomIndex(1)) = contour_area(inner_c);
    end
end
switch audioFlag
    case 1
        DaNew(:,unusedAtomIndex(1)) = Da(:,update_k);
    case 2
        DaNew(:,:,unusedAtomIndex(1)) = Da(:,:,update_k);
end


if ChangeFlag2, % change two atoms
    DvNew(:,:,frameStart:frameEnd,unusedAtomIndex(2)) = reshape(C(2,:),[dvr dvc frame]);
    if P.similarityArray(2)==99,
        for frameIndex = frameStart:frameEnd,
            tempdv = DvNew(:,:,frameIndex,unusedAtomIndex(2));
            inner_c = tempdv(23:38,:);
            DvNew(39,1,frameIndex,unusedAtomIndex(2)) = contour_area(inner_c);
        end
    end
    switch audioFlag
        case 1
            DaNew(:,unusedAtomIndex(2)) = Da(:,update_k);
        case 2
            DaNew(:,:,unusedAtomIndex(2)) = Da(:,:,update_k);
    end
    if dispMode,
        fprintf('Atom-%d is divided to Atom-%d and Atom-%d.\n', update_k,unusedAtomIndex(1),unusedAtomIndex(2));
    end
else
    if dispMode,
        fprintf('Atom-%d is divided to Atom-%d.\n', update_k,unusedAtomIndex(1));
    end
end

if ChangeFlagMore, % change more atoms
    [update_n_k ind] = max(Num_usedAtomIndex);
    update_k = usedAtomIndex(ind);
    index = atomIndex==update_k;
    y_array = ParamCoef.Translation_v(1,index);
    x_array = ParamCoef.Translation_v(2,index);
    z_array = ParamCoef.Translation_v(3,index);
    
    
    X = zeros(dvr,dvc,frame,update_n_k);
    for i = 1:update_n_k
        X(:,:,:,i) = Rv(y_array(i)+(0:dvr-1),x_array(i)+(0:dvc-1),z_array(i)+(frameStart-1:frameEnd-1));
    end
    X = reshape(X,[],update_n_k);
    X = X';
    
    % k-means are used to cluster the k-th atoms into 5 parts and
    % then update the atoms
    try
        [IDX, C] = kmeans(X, ChangeFlagMore);
    catch
        update_n_k = floor(update_n_k/ChangeFlagMore)*ChangeFlagMore;
        X = X(1:update_n_k,:);
        X = reshape(X,[],ChangeFlagMore,size(X,2));
        C = squeeze(mean(X));
    end
    
    for i = 1:ChangeFlagMore,
        DvNew(:,:,frameStart:frameEnd,unusedAtomIndex(2+i)) = reshape(C(i,:),[dvr dvc frame]);
        if P.similarityArray(2)==99,
            for frameIndex = frameStart:frameEnd,
                tempdv = DvNew(:,:,frameIndex,unusedAtomIndex(2+i));
                inner_c = tempdv(23:38,:);
                DvNew(39,1,frameIndex,unusedAtomIndex(2+i)) = contour_area(inner_c);
            end
        end
        switch audioFlag
            case 1
                DaNew(:,unusedAtomIndex(2+i)) = Da(:,update_k);
            case 2
                DaNew(:,:,unusedAtomIndex(2+i)) = Da(:,:,update_k);
        end
    end
    
    if dispMode,
        fprintf(['Atom-',num2str(update_k,'%d'),' is divided to Atom-' num2str(unusedAtomIndex(3:2+ChangeFlagMore),'%-20.5g'),'\n']);
    end
end

if CreateFlagMore,   % create more atoms, which seldom is used
    ChooseIndex = scanIndex;
    Index = find(ChooseIndex);
    Index = Index(randperm(length(Index)));
    dar = P.dar;
    for i=1:CreateFlagMore,
        idx = Index(i);
        [p, q, z] = ind2sub(size(scanIndex),idx);
        tempVideo = Rv(p+(0:dvr-1),q+(0:dvc-1),z+(frameStart-1:frameEnd-1));
        DvNew(:,:,frameStart:frameEnd,unusedAtomIndex(2+i))=tempVideo;
        
        audio_start = round((z-1)*FsaVsFsv)+1;
        switch audioFlag
            case 1
                temp = Ra(audio_start+(0:dar-1));
                temp = temp./norm(temp(:));
                DaNew(:,unusedAtomIndex(2+i)) = temp;
            case 2
                temp = Ra(:,audio_start+(0:dar-1));
                temp = temp./norm(temp(:));
                DaNew(:,:,unusedAtomIndex(2+i)) = temp;
        end
    end
    
    if dispMode,
        fprintf(['Create new atoms-' num2str(unusedAtomIndex(3:2+CreateFlagMore),'%-20.5g'),'\n']);
    end
end


