function [ParamCoef Ra Rv ParamsNew audioRev videoRev] = coding(audio, video, P)
% Qingju Liu, University of Surrey
% sparse coding for the audio-visual data (audio video) with pre-defined
% dictionary (Da,Dv); The video atoms are not overlapped (spacial sparse)
% and the audio atoms overlap with each other.
% N audio-visual atoms are used to approximate the audio-visual data;

if any(P.hopSize~=1),
    if ~isempty(P.threshold),
        P.threshold = P.threshold*0.9;
    end
    if ~isempty(P.realThreshold),
        P.realThreshold = P.realThreshold*0.9;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% The atom projection parameter structure atomP
AtomP = atomProjectPara(P);

audioFlag = P.audioFlag;
Nk = P.atomNum;
N = P.atomsUsedNum;
Da = P.Da;
Dv = P.Dv;
yHop = P.hopSize(1);
xHop = P.hopSize(2);
zHop = P.hopSize(3);
ySearch = P.yxzSearch(1);
xSearch = P.yxzSearch(2);
zSearch = P.yxzSearch(3);
dvr = P.sizeVideoAtom(1);
dvc = P.sizeVideoAtom(2);
frame = P.sizeVideoAtom(3);
noOverlap = P.noOverlap;
[Vr Vc Frame] = size(video);
scanIndex = P.scanIndex;
dar = P.dar;
projectionIndex = P.projectionIndex;
similarityIndex = P.similarityIndex;
paramsOld = P.paramsOld;


timeCons = zeros(1,Frame);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Dictionary projection in the whole area
%%% AtomP.da AtomP.dv AtomP.projectionIndex AtomP.paramsOld
%%% AtomP.affectedCoordinates AtomP.maxCa

Params = cell(1,Nk);
AtomP.affectedCoordinates = [1 1 1; P.yxzSearch];
for k = 1:Nk,
    switch P.audioFlag,
        case 1
            AtomP.da = Da(:,k);
        case 2
            AtomP.da = Da(:,:,k);
    end
    AtomP.dv = Dv(:,:,:,k);
    AtomP.projectionIndex = projectionIndex(:,k);
    AtomP.similarityIndex = similarityIndex(:,k);
    if ~isempty(paramsOld),
        AtomP.paramsOld = paramsOld{k};
    else
        AtomP.paramsOld = [];
    end
    AtomP.u_dv = P.uDv(k);
    AtomP.sigma_dv_sqr = P.sigmaDv_sqr(k);
    %     tic
    [tempParams, AtomP]= atomProjection(audio,video,AtomP);
    %     toc
    Params{k} = tempParams;
end
ParamsNew = Params;
AtomP.similarityIndex = [0;0];

if ~isempty(AtomP.realThreshold),
    realThresholdValue_a = AtomP.realThreshold(1);
    realThresholdValue_v = AtomP.realThreshold(2);
else
    threshold_a = AtomP.threshold(1);
    threshold_v = AtomP.threshold(2);
    maxCa = AtomP.maxCa;
    maxCv = AtomP.maxCv;
    realThresholdValue_a = threshold_a*maxCa;
    realThresholdValue_v = threshold_v*maxCv;
end


Ra = audio;
Rv = video;
videoRev = zeros(size(video));
% to store the coding Parameters.
atomIndex = zeros(1,N);     % the k_n-th atom is chosen in the n-th iteration
Translation_a = zeros(1,N); % frame delay for the n-th allocated atom
Translation_v =  zeros(3,N);% ditto
aCoef =  zeros(1,N);        % c_n
CaCoef =  zeros(1,N);       % audio-simialgiry
CvCoef =  zeros(1,N);      	% visual similarity
CavCoef =  zeros(1,N);      % AV similarity
RealNum = 0; % the real number of atom used in the approximation


MatC = zeros([P.yxzSearch, Nk]);
for  k = 1:Nk,
        tempCa = Params{k}.Ca;
        tempCv = Params{k}.Cv;
        tempCav = Params{k}.Cav;
        tempIndexa = tempCa<realThresholdValue_a;
        tempIndexv = tempCv<realThresholdValue_v;
        tempCav(:,:,tempIndexa) = 0;
        tempCav(tempIndexv) = 0;
        MatC(:,:,:,k) = tempCav;
end

for i = 1:N,
    %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Selection step: Based on a similarity criterion between the current
    % residual and dictionary elements, the best matching atom is selected
    % for the next projection step
    [C_i idx] = max(MatC(:));
    
    if C_i<=0, break; end
    
    [p_i, q_i, z_i, k_i] = ind2sub(size(MatC),idx);
    
    switch audioFlag,
        case 1
            da_i = Da(:,k_i);
        case 2
            da_i = Da(:,:,k_i);
    end
    dv_i = Dv(:,:,:,k_i);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % To get more accurate result, calculate when step_size==1 near the
    % maximum points the the k_i -th atom.
    if any(P.hopSize~=1)
        % redefine the affected area that needs to be refined:
        subScanYstart = max(1,p_i-yHop+1);
        subScanYend = min(p_i+yHop-1,Vr-dvr+1);
        subScanXstart = max(1,q_i-xHop+1);
        subScanXend = min(q_i+xHop-1,Vc-dvc+1);
        subScanZstart = max(1,z_i-zHop+1);
        subScanZend = min(z_i+zHop-1,Frame-frame+1);
        affectedCoordinates = [subScanYstart subScanXstart subScanZstart; subScanYend subScanXend subScanZend];
        
        AtomP.hopSize = [1 1 1];
        %%% AtomP.da AtomP.dv AtomP.projectionIndex AtomP.paramsOld
        %%% AtomP.affectedCoordinates AtomP.maxCa
        AtomP.da = da_i;
        AtomP.dv = dv_i;
        AtomP.projectionIndex = [1;1];  % both Ca and Cv should be calculated.
        AtomP.paramsOld = Params{k_i};
        AtomP.u_dv = P.uDv(k_i);
        AtomP.sigma_dv_sqr = P.sigmaDv_sqr(k_i);
        AtomP.affectedCoordinates = affectedCoordinates;
        
        tempParams= atomProjection(Ra,Rv,AtomP);
        Params{k_i} = tempParams;
        
        subCa = Params{k_i}.Ca(subScanZstart:subScanZend);
        subCv = Params{k_i}.Cv(subScanYstart:subScanYend,subScanXstart:subScanXend,subScanZstart:subScanZend);
        subCav =Params{k_i}.Cav(subScanYstart:subScanYend,subScanXstart:subScanXend,subScanZstart:subScanZend);
        subIndexa = subCa<realThresholdValue_a;
        subIndexv = subCv<realThresholdValue_v;
        subCav(:,:,subIndexa) = 0;
        subCav(subIndexv) = 0;
        [C_i idx] = max(subCav(:));
        [subP_i, subQ_i, subZ_i] = ind2sub(size(subCav),idx);
        p_i = subScanYstart+subP_i-1;
        q_i = subScanXstart+subQ_i-1;
        z_i = subScanZstart+subZ_i-1;
    end
    
    alpha_i = Params{k_i}.ALPHA(z_i);
    
    RealNum = RealNum+1;
    
    % Calculate the residue
    coefa_i = Params{k_i}.Coefa(z_i);
    audio_start = round((z_i-1)*P.FsaVsFsv)+alpha_i;
    switch audioFlag,
        case 1
            Ra(audio_start+(0:dar-1)) = Ra(audio_start+(0:dar-1))-coefa_i*da_i';
        case 2
            Ra(:,audio_start+(0:dar-1)) = Ra(:,audio_start+(0:dar-1))-coefa_i*da_i;
    end
    
    atomIndex(i) =  k_i;            % the k_i-th atom is chosen in the i-th iteration
    Translation_a(i) = alpha_i;     % frame delay for the n-th allocated atom
    Translation_v(:,i) = [p_i;q_i;z_i];
    aCoef(i) =  coefa_i;
    CaCoef(i) =  Params{k_i}.Ca(z_i);               % audio-simialgiry
    CvCoef(i) =  Params{k_i}.Cv(p_i,q_i,z_i);       % visual-similarity
    CavCoef(i) = C_i;                               % AV-similarity
    
    
    
    %% Before continue to find the next optimum atom, Ca, Cav in the affected area need to be recalculated.
        
        % At each time frame, at most 3 signals are allowed to be located
        timeCons(z_i+(0:frame-1)) = timeCons(z_i+(0:frame-1))+1;
        temp = timeCons(z_i+(0:frame-1));
        findIndex = find(temp>=3);
        if ~isempty(findIndex),
            index = z_i+findIndex-1;
            aIndex = repmat(index(:),[1 2*frame-1])+repmat((1-frame:frame-1),[length(index) 1]);
            aIndex(aIndex<1)=[];
            aIndex(aIndex>size(scanIndex,3)) = [];
            scanIndex(:,:,aIndex(:)) = false;
        end
        
        
        % Another AV atom is not allowed to be allocated in the noOverlap defined as follows
        if ~isempty(noOverlap),
            Y_a = noOverlap(1);
            X_a = noOverlap(2);
            Z_a = noOverlap(3);
            affctedY_start2 = max(1,p_i-Y_a); affctedY_end2 = min(p_i+Y_a,Vr-dvr+1);
            affctedX_start2 = max(1,q_i-X_a); affctedX_end2 = min(q_i+X_a,Vc-dvc+1);
            affctedZ_start2 = max(1,z_i-Z_a); affctedZ_end2 = min(z_i+Z_a,Frame-frame+1);
            scanIndex(affctedY_start2:affctedY_end2, affctedX_start2:affctedX_end2,affctedZ_start2:affctedZ_end2) = false;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%%%%%%%%%% In the affected time the Ca will be recalculated
        affctedZ_start = max(1,z_i-frame+1); affctedZ_end = min(z_i+frame-1,Frame-frame+1);
        affctedZ_start = floor((affctedZ_start-1)/zHop)*zHop+1; % the same grid (where Cv is computed)as in the full projection
        for k = 1:Nk,
            Params{k}.Ca(affctedZ_start:affctedZ_end) = 0; 
            Params{k}.Cav(:,:,affctedZ_start:affctedZ_end) = 0;  
        end
        
        %% Dictionary projection in the affected area
        % update the similarity with the residue via projection. (Only Audio Similarity is recalculated)
        AtomP.affectedCoordinates = [1,1,affctedZ_start;ySearch,xSearch,affctedZ_end];
        AtomP.hopSize = P.hopSize;          % hopSize as the grid
        AtomP.scanIndex = scanIndex;        % Projected to scanIndex
        AtomP.projectionIndex = [1;0];      % only audio projection
        for k = 1:Nk,
            switch audioFlag,
                case 1
                    AtomP.da = Da(:,k);
                case 2
                    AtomP.da = Da(:,:,k);
            end
            AtomP.dv = Dv(:,:,:,k);
            AtomP.u_dv = P.uDv(k);
            AtomP.sigma_dv_sqr = P.sigmaDv_sqr(k);
            AtomP.paramsOld = Params{k};
            tempParams = atomProjection(Ra,Rv,AtomP);
            Params{k} = tempParams;
            
            tempCa = tempParams.Ca(affctedZ_start:affctedZ_end);
            tempCv = tempParams.Cv(:,:,affctedZ_start:affctedZ_end);
            tempCav = tempParams.Cav(:,:,affctedZ_start:affctedZ_end);
            tempIndexa = tempCa<realThresholdValue_a;
            tempIndexv = tempCv<realThresholdValue_v;
            tempCav(:,:,tempIndexa) = 0;
            tempCav(tempIndexv) = 0;
            MatC(:,:,affctedZ_start:affctedZ_end,k) = tempCav;
        end

    
    if any(scanIndex(:))
    else
        break;
    end
end


atomIndex =  atomIndex(1:RealNum);
Translation_a = Translation_a(1:RealNum);
Translation_v = Translation_v(:,1:RealNum);
aCoef = aCoef(1:RealNum);
CaCoef =  CaCoef(1:RealNum);
CvCoef =  CvCoef(1:RealNum);
CavCoef = CavCoef(1:RealNum);

if any(P.hopSize~=1),
    realThresholdValue_a = realThresholdValue_a/0.9;
    realThresholdValue_v = realThresholdValue_v/0.9;
    tempIndexa = CaCoef>realThresholdValue_a;
    tempIndexv = CvCoef>realThresholdValue_v;
    tempIndex = tempIndexa & tempIndexv;
    
    for i = find(~tempIndex),
        % update the residue
        coefa_i = aCoef(i);
        k_i = atomIndex(i);
        alpha_i = Translation_a(i);
        z_i =  Translation_v(3,i);
        audio_start = round((z_i-1)*P.FsaVsFsv)+alpha_i;
        switch audioFlag,
            case 1
                da_i =  Da(:,k_i);
                Ra(audio_start+(0:dar-1)) = Ra(audio_start+(0:dar-1))+coefa_i*da_i';
            case 2
                da_i =  Da(:,:,k_i);
                Ra(:,audio_start+(0:dar-1)) = Ra(:,audio_start+(0:dar-1))+coefa_i*da_i;
        end
        
    end
    
    atomIndex =  atomIndex(tempIndex);
    Translation_a = Translation_a(tempIndex);
    Translation_v = Translation_v(:,tempIndex);
    aCoef = aCoef(tempIndex);
    CaCoef =  CaCoef(tempIndex);
    CvCoef =  CvCoef(tempIndex);
    CavCoef = CavCoef(tempIndex);
end


if P.dispMode,
    usedAtomIndex = unique(atomIndex);
    atomNum = length(usedAtomIndex);
    nn = histc(atomIndex,usedAtomIndex);%frequency
    TotalAtomNumber = length(atomIndex);
    
    [nn index] = sort(nn,'descend');
    usedAtomIndex = usedAtomIndex(index);
    
    fprintf(['\n ', num2str(atomNum), ' atoms (', num2str(usedAtomIndex,'%-5.5g'),...
        ') are used for coding, located at ', num2str(TotalAtomNumber),...
        ' locations (', num2str(nn,'%-5.5g'),'): \n'])
    fprintf(['Indices    \t',num2str(atomIndex,'%-20.5g'),'\n\n']);
    fprintf(['Positions x\t',num2str(Translation_v(1,:),'%-20.5g'),'\n']);
    fprintf(['          y\t',num2str(Translation_v(2,:),'%-20.5g'),'\n']);
    fprintf(['          f\t',num2str(Translation_v(3,:),'%-20.5g'),'\n\n']);
    fprintf(['Ca         \t',num2str(CaCoef,'%-20.5g'),'\n']);
    fprintf(['Cv         \t',num2str(CvCoef,'%-20.5g'),'\n']);
    fprintf(['Cav        \t',num2str(CavCoef,'%-20.5g'),'\n']);
end

%%%%%%%%%%%% ParamCoef.FsaVsFsv is useless except for the time domain
%%%%%%%%%%%% display of audio data
ParamCoef.FsaVsFsv = P.FsaVsFsv;


ParamCoef.atomIndex =  atomIndex;
ParamCoef.Translation_a = Translation_a;
ParamCoef.Translation_v = Translation_v;
ParamCoef.aCoef = aCoef;
ParamCoef.CaCoef =  CaCoef;
ParamCoef.CvCoef =  CvCoef;
ParamCoef.CavCoef = CavCoef;


if nargout>4,
    audioRev = audio - Ra;
    videoRev = zeros(size(video));
    
    frameStart = P.frameStartEnd(1);
    frameEnd= P.frameStartEnd(2);
    
    for i = 1:length(atomIndex),
        k_i = atomIndex(i);
        dv_i = Dv(:,:,:,k_i);
        p_i =  Translation_v(1,i);
        q_i =  Translation_v(2,i);
        z_i =  Translation_v(3,i);
        
        videoRev(p_i:p_i+dvr-1,q_i:q_i+dvc-1,z_i+frameStart-1:z_i+frameEnd-1) = dv_i(:,:,frameStart:frameEnd); 
        
    end
end

end

