%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% update ParamCoef, delete the atoms that appears too few times
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ParamCoefNew Ra Rv] = deleteAtomsTooFew(ParamCoef, Ra, Rv, P, MonaciFlag)
% Qingju Liu, University of Surrey
if nargin<5,
    MonaciFlag = 0;         % default is not Monaci's method
end
Da = P.Da;
Dv = P.Dv;
audioFlag = P.audioFlag;
thresholdNum = P.thresholdNum;
dar = P.dar;
dvr = P.dvr;
dvc = P.dvc;
frame = P.frame;
FsaVsFsv = P.FsaVsFsv;
Nk = P.atomNum;

atomIndex = ParamCoef.atomIndex;
usedAtomIndex = unique(atomIndex);
nn = histc(atomIndex,usedAtomIndex);%frequency

if isempty(thresholdNum),
    threshold1 = floor(length(atomIndex)/(3*Nk));
    threshold2 = round(length(atomIndex)/(4*length(usedAtomIndex)));
    thresholdNum = max([2 threshold1,threshold2]);
end

temp = find(nn<thresholdNum);
if ~isempty(temp),
    AtomFew = usedAtomIndex(temp);
    % 1 Not choose,
    NotChoose = ismember(atomIndex,AtomFew);
else
    NotChoose = ismember(atomIndex,[]);
end


for i = find(NotChoose),
    k = atomIndex(i);
    p = ParamCoef.Translation_v(1,i);
    q = ParamCoef.Translation_v(2,i);
    z = ParamCoef.Translation_v(3,i);
    alpha = ParamCoef.Translation_a(i);
    
    coef_a = ParamCoef.aCoef(i);
    audio_start = round((z-1)*FsaVsFsv)+alpha;
    switch audioFlag,
        case 1
            Ra(audio_start+(0:dar-1)) = ...
                Ra(audio_start+(0:dar-1))+coef_a*Da(:,k)';
        case 2
            Ra(:,audio_start+(0:dar-1)) = ...
                Ra(:,audio_start+(0:dar-1))+coef_a*Da(:,:,k);
    end

    if MonaciFlag,
        coef_v = ParamCoef.vCoef(i);
        frameStart = P.frameStartEnd(1);
        frameEnd = P.frameStartEnd(2);
        Rv(p:p+dvr-1,q:q+dvc-1,z+frameStart-1:z+frameEnd-1) = ...
            Rv(p:p+dvr-1,q:q+dvc-1,z+frameStart-1:z+frameEnd-1)+coef_v*Dv(:,:,frameStart:frameEnd,k);
    end
end


ParamCoefNew = ParamCoef;
ParamCoefNew.atomIndex(NotChoose) =  [];
ParamCoefNew.Translation_a(NotChoose) = [];
ParamCoefNew.Translation_v(:,NotChoose) = [];
ParamCoefNew.aCoef(NotChoose) = [];
ParamCoefNew.CaCoef(NotChoose) = [];
if MonaciFlag,ParamCoefNew.vCoef(NotChoose) = [];end
ParamCoefNew.CvCoef(NotChoose) = [];
ParamCoefNew.CavCoef(NotChoose) = [];