function [ProjectionIndex SimilarityIndex] = detectProjectionIndex(DaOld,Da,DvOld,Dv,P)
% QINGJU LIU, University of Surrey
% [1 1] full matching pursuit (audio and visual atoms are different, dvOld~=dv daOld~=da)
% [1 0] matching pursuit only for the audio part (visual atoms are the same dvOld=dv)
% [0 0] use the MP coefficents of the previous iteration (dvOld=dv daOld=da)
Nk = P.atomNum;
ProjectionIndex = ones(2,Nk);
SimilarityIndex = zeros(2,Nk);
if ~isempty(DaOld) && ~isempty(DvOld),
    frameStart = P.frameStartEnd(1);
    frameEnd = P.frameStartEnd(2);
    % reshape the dictionary for comparision
    DaOld = reshape(DaOld,[],Nk);
    Da= reshape(Da,[],Nk);
    DvOld = DvOld(:,:,frameStart:frameEnd,:);
    Dv = Dv(:,:,frameStart:frameEnd,:);
    DvOld = reshape(DvOld,[],Nk);
    Dv= reshape(Dv,[],Nk);
    
    
    for k = 1:Nk, % the k-th atom
        
        daOld = DaOld(:,k);
        da = Da(:,k);
        norm_diffa = norm(daOld-da,2);
        if norm_diffa<0.05,
            ProjectionIndex(1,k)=0;
        end
        if norm_diffa<0.1,
            SimilarityIndex(1,k)=1;
        end
        
        dvOld = DvOld(:,k);
        dv = Dv(:,k);
        norm_diffv = norm(dvOld-dv,2);
        norm_dv = norm(dv,2);
        if norm_diffv<0.01*norm_dv,
            ProjectionIndex(2,k)=0;
        end
        if norm_diffv<0.1*norm_dv,
            SimilarityIndex(2,k)=1;
        end
        
    end
end
end

