function [Da,Dv] = energyInCentre(Da,Dv,P)
% Qingju Liu, University of Surrey
Nk = P.atomNum; 
audioFlag = P.audioFlag; 
FsaVsFsv = P.FsaVsFsv;
dar = P.dar;
frame = P.sizeVideoAtom(3);


for k = 1:Nk,
    switch audioFlag,
        case 1
            da = Da(:,k);
        case 2
            da = Da(:,:,k);
    end
    dv = Dv(:,:,:,k);
    audio_centre = obtainEnergyCentre(da);
    low_ratio = 0.4;high_ratio = 0.6;
    % time translation
    if audio_centre>(low_ratio*dar) && audio_centre<(high_ratio*dar),
        % do nothing
    elseif  frame>=6,
        switch audioFlag,
            case 1
                da = mydelay(da,round(dar/2-audio_centre));
                Da(:,k) = da;
            case 2
                da = mydelay(da,round(dar/2-audio_centre),2);
                Da(:,:,k) = da;
        end
        frame_centre = audio_centre/FsaVsFsv;
        dv = mydelay2(dv, round(frame/2-frame_centre),3);
        Dv(:,:,:,k) = dv;
    end
    
end







function centre = obtainEnergyCentre(da)
if numel(da) ~= length(da)
    da = sum(da.*conj(da)); % spectrum to power spectrum
end
dar = length(da);
% calculate the audio energy centre;
temp = [1:dar]-0.5; 
weis = abs(da(:));
centre = temp*weis/sum(weis);





function delayM = mydelay(M,T,dim)
% Qingju Liu, university of surrey
% This programme dealy matrix M in the 'dim'-the dimension
if nargin<3,
    dim = 1;
end
A = size(M);
if dim>length(A),
    error(sprintf('There is no dimension %d !', dim));
end
if isvector(M),
    dim = 1; M = M(:);
end
if dim>1,
    dim_before = prod(A(1:(dim-1)));
else
    dim_before = 1;
end
if dim<length(A),
    dim_after = prod(A(dim+1:end));
else
    dim_after = 1;
end

delayM = zeros(dim_before,A(dim),dim_after);
M = reshape(M,[dim_before,A(dim),dim_after]);
if T>=0,
    if (T+1)<=A(dim),
        delayM(:,(T+1):A(dim),:) = M(:,1:(A(dim)-T),:);
    end
elseif T<0
    if 1-T<=A(dim),
        delayM(:,1:(A(dim)+T),:) = M(:,1-T:A(dim),:);
    end
end

delayM = reshape(delayM,A);


function delayM = mydelay2(M,T,dim)
% Qingju Liu, university of surrey
% This programme dealy matrix M in the 'dim'-the dimension
if nargin<3,
    dim = 1;
end
A = size(M);
if dim>length(A),
    error(sprintf('There is no dimension %d !', dim));
end
if isvector(M),
    dim = 1; M = M(:);
end
if dim>1,
    dim_before = prod(A(1:(dim-1)));
else
    dim_before = 1;
end
if dim<length(A),
    dim_after = prod(A(dim+1:end));
else
    dim_after = 1;
end

% delayM = zeros(dim_before,A(dim),dim_after);
M = reshape(M,[dim_before,A(dim),dim_after]);
if T>=0,
    %% difference here
    delayM = repmat(M(:,1,:),[1 A(dim) 1]);
    if (T+1)<=A(dim),
        delayM(:,(T+1):A(dim),:) = M(:,1:(A(dim)-T),:);
    end
elseif T<0
    %% difference here
    delayM = repmat(M(:,A(dim),:),[1 A(dim) 1]);
    if 1-T<=A(dim),
        delayM(:,1:(A(dim)+T),:) = M(:,1-T:A(dim),:);
    end
end

delayM = reshape(delayM,A);


























% function [Da,Dv] = EnergyInCentre(Da,Dv)
% % Qingju Liu, University of Surrey
% if length(size(Dv))==3, % only one dictionary atom:)
%     [dvr dvc frame] = size(Dv);
%     Nk=1;
%     if numel(Da) == length(Da),   % 1 D
%         audioFlag = 1;
%         dar = length(Da);
%     else                                % 2 D
%         audioFlag = 2;
%         [halfNFFT,dar]  = size(Da);
%     end
% else                    % Nk dictionary atom
%     [dvr dvc frame,Nk] = size(Dv);
%     if length(size(Da))==2,
%         audioFlag = 1;
%         [dar,Nk1]  = size(Da);
%     else
%         audioFlag = 2;
%         [halfNFFT,dar,Nk1]  = size(Da);
%     end
%     assert(Nk1==Nk, '# audio atoms should equal to # visual atoms !'); clear Nk1
% end
% 
% for k = 1:Nk,
%     switch audioFlag,
%         case 1
%             da = Da(:,k);
%         case 2
%             da = Da(:,:,k);
%     end
%     dv = Dv(:,:,:,k);
%     %     % get the dynamic difference of the visual atom
%     %     dv = filter([1 -1],1,dv,[],3); dv(:,:,1) = 0;
%     [audio_centre,y_centre,x_centre,frame_centre] = obtainEnergyCentre(da,dv);
%     %     dv = Dv(:,:,:,k);
%     low_ratio = 0.4;high_ratio = 0.6;
%     % time translation
%     if audio_centre>(low_ratio*dar) && audio_centre<(high_ratio*dar) && frame>=6,
%         % do nothing
%     else
%         switch audioFlag,
%             case 1
%                 da = mydelay(da,round(dar/2-audio_centre));
%             case 2
%                 da = mydelay(da,round(dar/2-audio_centre),2);
%         end
%         dv = mydelay(dv, round(frame/2-frame_centre),3);
%     end
%     % xy-axis translation
%     if y_centre>(low_ratio*dvr) && y_centre<(high_ratio*dvr) && dvr>=6,
%     else
%         dv = mydelay(dv, round(dvr/2-y_centre),1);
%     end
%     if x_centre>(low_ratio*dvc) && x_centre<(high_ratio*dvc) && dvc>=6,
%     else
%         dv = mydelay(dv, round(dvc/2-x_centre),2);
%     end
%     switch audioFlag,
%         case 1
%             Da(:,k) = da;
%         case 2
%             Da(:,:,k) = da;
%     end
%     Dv(:,:,:,k) = dv;
% end
% 
% 
% function [audio_centre,y_centre,x_centre,frame_centre] = obtainEnergyCentre(da,dv)
% if numel(da) ~= length(da)
%     da = sum(abs(da));
% end
% dar = length(da);
% [dvr dvc frame] = size(dv);
% % calculate the audio energy centre;
% temp = [1:dar]-0.5; weis = abs(da(:)).^2;
% audio_centre_temp = temp*weis/sum(weis);
% % calculate the video 'energy' centre;
% temp = [1:dvr]-0.5; weis = sum(sum((abs(dv)).^2,3),2); weis = weis(:);
% y_centre = temp*weis/sum(weis);
% temp = [1:dvc]-0.5; weis = sum(sum((abs(dv)).^2,3),1);weis = weis(:);
% x_centre = temp*weis/sum(weis);
% temp = [1:frame]-0.5; weis = sum(sum((abs(dv)).^2,2),1);weis = weis(:);
% frame_centre_temp = temp*weis/sum(weis);
% time_centre = 0.5*(audio_centre_temp/dar + frame_centre_temp/frame);
% audio_centre = time_centre*dar;
% frame_centre = time_centre*frame;
% 
% 
% function delayM = mydelay(M,T,dim)
% % Qingju Liu, university of surrey
% % This programme dealy matrix M in the 'dim'-the dimension
% if nargin<3,
%     dim = 1;
% end
% A = size(M);
% if dim>length(A),
%     error(sprintf('There is no dimension %d !', dim));
% end
% if isvector(M),
%     dim = 1; M = M(:);
% end
% if dim>1,
%     dim_before = prod(A(1:(dim-1)));
% else
%     dim_before = 1;
% end
% if dim<length(A),
%     dim_after = prod(A(dim+1:end));
% else
%     dim_after = 1;
% end
% 
% delayM = zeros(dim_before,A(dim),dim_after);
% M = reshape(M,[dim_before,A(dim),dim_after]);
% if T>0,
%     if (T+1)<=A(dim),
%         delayM(:,(T+1):A(dim),:) = M(:,1:(A(dim)-T),:);
%     end
% elseif T<0
%     if 1-T<=A(dim),
%         delayM(:,1:(A(dim)+T),:) = M(:,1-T:A(dim),:);
%     end
% end
% 
% delayM = reshape(delayM,A);
