function [scanIndex visualScan audioScan] =  initialScanIndex(audio, video, sizeVideoAtom, frameDynamic, threshold, useAreaFrame)
% Qingju Liu
% input:
%   useAreaFrame:       Pre-defination of the frames being scanned.
%   Mx2 matrix. e.g.    [5 10] or [5 10; 30 40]
%   frameDynamic:      for calculation of dynamic visual features

%
% output
%   scanIndex:      the logic matrix associated with each pixel
%   VisualScan:     the variance of the temporary videoAtom-sized video part
%   audioScan:      the audio 'energy' for weighting


if nargin<3,
    error('At least three inputs are needed')
end

if numel(audio) == length(audio),   % 1 D
    audio = abs(audio);
else                                % 2 D---->1 D
    audio = sum(abs(audio));
end
audio_length = length(audio);
[Vr Vc Frame] = size(video);
dvr = sizeVideoAtom(1);
dvc = sizeVideoAtom(2);
frame = sizeVideoAtom(3);
FsaVsFsv = audio_length/Frame;
dar = round(FsaVsFsv*frame);
ySearch = Vr-dvr+1;
xSearch = Vc-dvc+1;
zSearch = Frame-frame+1;

if nargin<6,
    useAreaFrame = [1 Frame];% use all the frames
end

if nargin>4 && ~isempty(threshold),
    assert(threshold>=0 && threshold<=1, 'Error!')
else
    threshold=0.01;
end
if nargin>3 && ~isempty(frameDynamic),
    fff = frameDynamic;
else
    fff = 10*frame;
end
% calculate the dynamic visual feature
dVideo = videoTemporalWhitening(video,0.8,fff);
dVideo = abs(dVideo);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% First obtain the audioScan
audioScan = zeros(1,zSearch);
for tBlock = 1:size(useAreaFrame,1)     % for each snipplet
    frameStart = useAreaFrame(tBlock,1);
    frameEnd = useAreaFrame(tBlock,2);
    audioStart = round((frameStart-1)*FsaVsFsv)+1;
    audioEnd = round(frameEnd*FsaVsFsv);
    subAudio = audio(audioStart:audioEnd);
    N = (frameEnd-frameStart-frame+2);
    Index = round((0:N-1)*FsaVsFsv); Index(end) = Index(end)-1;
    Index = repmat(Index,[dar,1])+repmat((1:dar)',[1 N]);
    subAudio = subAudio(Index);
    subAudio = sum(subAudio.^2);
    meanValue = trimmean(subAudio,10);
    threshold1 = 0.1*meanValue;
    [N,X] = hist(subAudio,10);
    threshold2 = X(1);
    if threshold2<threshold1,
        thresholdA = threshold1;
    else
        thresholdA = threshold1*2;
    end
    
    subAudio(subAudio<thresholdA) = 0;
    temp = frameEnd-frameStart-frame+1;
    audioScan(frameStart+(0:temp)) = subAudio;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Then obtain the visualScan
% for the memory Problem
M = zeros(ySearch,xSearch,zSearch);
b3 = ones(1,fff);
for tBlock = 1:size(useAreaFrame,1)     % for each snipplet
    frameStart = useAreaFrame(tBlock,1);
    frameEnd = useAreaFrame(tBlock,2);
    subdVideo = dVideo(:,:,frameStart:frameEnd);
    subframeUseN = frameEnd-frameStart+1;
    if subframeUseN>fff,
        subdVideo = filter(b3,1,subdVideo,[],3);
        subdVideo = subdVideo(:,:,fff:end);
        if fff>frame
            catdVideo = repmat(subdVideo(:,:,1),[1 1 fff-frame]);
            subdVideo = cat(3,catdVideo,subdVideo);
        elseif fff<frame
            subdVideo = subdVideo(:,:,(frame-fff+1):end);
        end
    else
        subdVideo = repmat(mean(subdVideo,3),[1 1 subframeUseN-frame+1]);
    end
    b1 = ones(1,dvr);
    subdVideo = filter(b1,1,subdVideo,[],1);
    subdVideo = subdVideo(dvr:end,:,:);
    b2 = ones(1,dvc);
    subdVideo = filter(b2,1,subdVideo,[],2);
    subdVideo = subdVideo(:,dvc:end,:);
    
    subframeUseN = frameEnd-frameStart-frame+2;
    % To avoid the outliers, we further process the scanning weights
    subdVideo = reshape(subdVideo,[],subframeUseN);
    temp = max(subdVideo,[],1);
    indexFlag = subdVideo<repmat(reshape(0.8*temp,[1 subframeUseN]),[round(ySearch*xSearch),1]);
    subdVideo(indexFlag) = 0;
    subdVideo = reshape(subdVideo,[ySearch,xSearch,subframeUseN]);
    M(:,:,frameStart+(0:subframeUseN-1)) = subdVideo;
end

visualScan = M;
audioScan(audioScan>0) = 1;
AudioVisualScan = repmat(reshape(audioScan,[1 1 zSearch]),[ySearch xSearch 1]) .* visualScan;

anotherThreshold = trimmean(temp,30);

thresholdValue = anotherThreshold*threshold;
scanIndex = AudioVisualScan>thresholdValue;
scanIndex(:,:,end) = false;