function [Da,Dv,ParamCoef,Ra,Rv] = learning_Monaci(ParamCoef, Ra, Rv, P)
% Qingju Liu, University of Surrey
% This is the updats of the atoms after coding stage
% Visual atoms are updated via calculate of the mean value of the visual
% kernals. audio atoms are updated via K-SVD;

Da = P.Da;
Dv = P.Dv;
FsaVsFsv = P.FsaVsFsv;
audioFlag = P.audioFlag;
halfNFFT = P.halfNFFT;
dar = P.dar;
frameStart = P.frameStartEnd(1);
frameEnd = P.frameStartEnd(2);
frameUse = frameEnd-frameStart+1;

[dvr,dvc,frame,Nk] = size(Dv);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% update ParamCoef, delete the atoms that appears too few times
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ParamCoef Ra Rv] = deleteAtomsTooFew(ParamCoef,Ra,Rv,P,1);
atomIndex = ParamCoef.atomIndex;
usedAtomIndex = unique(atomIndex);
nn = histc(atomIndex,usedAtomIndex);%frequency

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(usedAtomIndex),
    atomOrder = randperm(length(usedAtomIndex));
    
    for kk = atomOrder,
        k = usedAtomIndex(kk); % update the k-th atom in the dictionary
        n_k = nn(kk);          % time of appearances for the k-th atom
        assert(n_k>1,'haha');
        index = find(atomIndex==k); % when/where the k-th atom occured
        
        % concatnate n_k audio signals where the k-th atoms occured
        % (where contributions of the other atoms are elimnated)
        switch audioFlag
            case 1
                tempAudio = zeros(dar,n_k);
            case 2
                tempAudio = zeros(halfNFFT,dar,n_k);
        end
        tempVideo = zeros(dvr,dvc,frameUse,n_k);
        j = 1;
        for i = index,
            p = ParamCoef.Translation_v(1,i);
            q = ParamCoef.Translation_v(2,i);
            z = ParamCoef.Translation_v(3,i);
            alpha = ParamCoef.Translation_a(i);
            coef_v = ParamCoef.vCoef(i);
            coef_a = ParamCoef.aCoef(i);
            audio_start = round((z-1)*FsaVsFsv)+alpha;
            switch audioFlag
                % add the contribution of the current audio atom
                case 1
                    tempAudio(:,j) = Ra(audio_start+(0:dar-1)) + coef_a*Da(:,k)';
                case 2
                    tempAudio(:,:,j) = Ra(:,audio_start+(0:dar-1)) + coef_a*Da(:,:,k);
            end
            % add the contribution of the current visual atom
            temp = Rv(p:p+dvr-1,q:q+dvc-1,z+frameStart-1:z+frameEnd-1)+coef_v*Dv(:,:,frameStart:frameEnd,k);
            tempVideo(:,:,:,j) = temp;
            j = j+1;
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%% core algorithm of KSVD
        sizeVec_a = size(tempAudio);
        tempAudio = reshape(tempAudio,[],n_k);
        [atom_a,s_a,gamma_a] = svds(tempAudio, 1);
        if audioFlag==2,
            atom_a = reshape(atom_a,halfNFFT,dar);
        end
        coefs = s_a*gamma_a;
        if sum(coefs)>0,
        else
            atom_a = -atom_a;
            coefs = -coefs;
        end
        tempAudio = reshape(tempAudio,sizeVec_a);
        
        % update Ra (audio residue)
        j = 1;
        for i = index
            z = ParamCoef.Translation_v(3,i);
            alpha = ParamCoef.Translation_a(i);
            audio_start = round((z-1)*FsaVsFsv)+alpha;
            switch audioFlag
                case 1
                    Ra(audio_start+(0:dar-1)) = ...
                        tempAudio(:,j) - coefs(j)*atom_a;
                case 2
                    Ra(:,audio_start+(0:dar-1)) = ...
                        tempAudio(:,:,j) - coefs(j)*atom_a;
            end
            j = j+1;
        end
        
        % Atom updation
        switch audioFlag
            case 1
                Da(:,k) = atom_a(:);
            case 2
                Da(:,:,k) = atom_a;
        end
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%% core algorithm of KSVD
        sizeVec_v = size(tempVideo);
        tempVideo = reshape(tempVideo,[],n_k);
        [atom_v,s_v,gamma_v] = svds(tempVideo, 1);
        atom_v = reshape(atom_v,dvr,dvc,frameUse);
        coefs = s_v*gamma_v;
        if sum(coefs)>0,
        else
            atom_v = -atom_v;
            coefs = -coefs;
        end
        tempVideo = reshape(tempVideo,sizeVec_v);
        
        % update Rv (visual residue)
        j = 1;
        for i = index
            p = ParamCoef.Translation_v(1,i);
            q = ParamCoef.Translation_v(2,i);
            z = ParamCoef.Translation_v(3,i);
            Rv(p:p+dvr-1,q:q+dvc-1,z+frameStart-1:z+frameEnd-1) = ...
                tempVideo(:,:,:,j) - coefs(j)*atom_v;
            j = j+1;
        end
        
        % Atom updation
        Dv(:,:,frameStart:frameEnd,k) = atom_v;
        
    end
end
P.Da = Da;
P.Dv = Dv;


if P.dispMode,
    atomIndex = ParamCoef.atomIndex;
    usedAtomIndex = unique(atomIndex);
    AtomNum = length(usedAtomIndex);
    nn = histc(atomIndex,usedAtomIndex);%frequency
    TotalAtomNumber = length(atomIndex);
    
    [nn index] = sort(nn,'descend');
    usedAtomIndex = usedAtomIndex(index);
    
    fprintf(['\n ', num2str(AtomNum), ' atoms (', num2str(usedAtomIndex,'%-5.5g'),...
        ') are converged, located at ', num2str(TotalAtomNumber),...
        ' locations (', num2str(nn,'%-5.5g'),'): \n'])
end

% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % comment this one when runing systhetic data
% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % % update unused atoms.
% if P.atomMinNumUse,   % conditional change atoms
%     [Da Dv] = changeAtoms(ParamCoef,Ra,Rv,P);
% end


% This function moves the most bimodal-informative part in each dictionary atom to the centre
[Da Dv] = energyInCentre(Da, Dv, P);

end
