function P = parameterCoding(audio, video, P)
%% First Define all the fields in the parameter structure P to avoid insufficient parameter for coding
if ~isfield(P, 'Da')||~isfield(P, 'Dv')
    error('Parameters contains at least the Dictionary!');
end
if isempty(P.Da) || isempty(P.Dv),
    error('Dictionary cannot be empty!');
end
Dv = P.Dv;
%% Optional Fields: P.audioFlag P.sizeAudioAtom P.sizeVideoAtom P.FsaVsFsv
% P.sparsity P.atomsUsedNum P.threshold P.noOverlap P.scanIndex
% P.similarityArray P.dispMode P.videoWeight P.paramsOld P.projectionIndex
[Vr Vc Frame] = size(video);
[dvr dvc frame Nk] = size(Dv);
%%% P.audioFlag
if isfield(P,'audioFlag') && ~isempty(P.audioFlag),
else
    if numel(audio) == length(audio),   % 1 D
        P.audioFlag = 1;
    else                                % 2 D
        P.audioFlag = 2;
    end
end

%%% P.atomNum
P.atomNum = Nk;

%%% P.sizeAudioAtom
if P.audioFlag==1,                  % 1D
    [dar,Nk] = size(P.Da);
    P.sizeAudioAtom = dar;
else                                % 2 D
    [halfNFFT,dar,Nk] = size(P.Da);
    P.sizeAudioAtom = [halfNFFT dar];
end
P.dar = dar;

%%% P.sizeVideoAtom
P.sizeVideoAtom = [dvr dvc frame];
P.dvr = dvr;P.dvc = dvc;P.frame = frame;

%%% P.FsaVsFsv
if isfield(P,'FsaVsFsv') && ~isempty(P.FsaVsFsv),
else
    if P.audioFlag==1,                  % 1D
        audioLength = length(audio);
    else
        audioLength = size(audio,2);
    end
    P.FsaVsFsv = audioLength/Frame;
end

%%% P.sparsity
if isfield(P,'sparsity') && ~isempty(P.sparsity),
else
    P.sparsity = 0.8;
end

%%% P.atomsUsedNum
if isfield(P,'atomsUsedNum') && ~isempty(P.atomsUsedNum),
else
    P.atomsUsedNum = round(P.sparsity*Frame);
end

%%% P.threshold and P.realThreshold, if P.realThreshold == [], then use P.Threshold;
if isfield(P,'threshold') && ~isempty(P.threshold),
else
    P.threshold = [0.000001;0.000001];
end

if isfield(P,'realThreshold'),
else
    P.realThreshold = [];
end

%%% P.noOverlap
if ~isfield(P,'noOverlap'),
    P.noOverlap = [];    % default: atoms may overlap with each other
end

%%% P.similarityArray
if isfield(P,'similarityArray') && ~isempty(P.similarityArray),
else
    P.similarityArray = [1 1];
end

%%% P.scanIndex
if isfield(P,'scanIndex') && ~isempty(P.scanIndex),
else
    if P.similarityArray(2)==6,
        frameDynamic = frame*2;
        scanIndex = InitialScanIndexLip(audio, video, P.sizeVideoAtom, frameDynamic, 0.1);
    else
        frameDynamic = frame;
        scanIndex = initialScanIndex(audio, video, P.sizeVideoAtom,frameDynamic,0.1);
    end
    P.scanIndex = scanIndex;
end




% P.similarityArray P.dispMode P.videoWeight P.paramsOld P.projectionIndex

%%% P.dispMode
if isfield(P,'dispMode') && ~isempty(P.dispMode),
else
    P.dispMode = 1;
end


%%% P.paramsOld
if ~isfield(P,'paramsOld'),
    P.paramsOld = [];
end

%%% P.projectionIndex
if isfield(P,'projectionIndex') && ~isempty(P.projectionIndex),
else
    P.projectionIndex = ones(2,Nk);
end

%%% P.similarityIndex
if isfield(P,'similarityIndex') && ~isempty(P.similarityIndex),
else
    P.similarityIndex = zeros(2,Nk);
end

%%% P.frameStartEnd
if isfield(P,'frameStartEnd') && ~isempty(P.frameStartEnd),
else
    P.frameStartEnd = [1 frame];
end

%%% P.yxzSearch
if isfield(P,'yxzSearch') && ~isempty(P.yxzSearch),
else
    ySearch = min(Vr-dvr+1,size(P.scanIndex,1));
    xSearch = min(Vc-dvc+1,size(P.scanIndex,2));
    zSearch = min(Frame-frame+1,size(P.scanIndex,3));
    P.yxzSearch = [ySearch xSearch zSearch];
end

%%% P.hopSize
if isfield(P,'hopSize') && ~isempty(P.hopSize),
else
    yHop = min(ySearch, max(1,round(P.dvr/10)));
    xHop = min(xSearch, max(1,round(P.dvc/10)));
    zHop = min(zSearch, max(1,round(P.frame/8)));
    P.hopSize = [yHop,xHop,zHop];
end

%%% P.nint
if isfield(P,'nint') && ~isempty(P.nint),
else
    P.nint = round(dar/frame);
end


%%% P.ExistingFigN
if isfield(P,'ExistingFigN') && ~isempty(P.ExistingFigN),
else
    P.ExistingFigN = length(findall(0,'Type','Figure'));
end


%%% P.u1 P.sigma1_sqr P.uDv P.sigmaDv_sqr
if any(ismember([2], P.similarityArray(2))), %%% using SSIM for visual similarity calculation
    if isfield(P,'u1') && ~isempty(P.u1) && isfield(P,'sigma1_sqr') && ~isempty(P.sigma1_sqr) ...
            && isfield(P,'uDv') && ~isempty(P.uDv) && isfield(P,'sigmaDv_sqr') && ~isempty(P.sigmaDv_sqr),
    else
        scanIndex = P.scanIndex;
        u1 = zeros(size(scanIndex));
        sigma1_sqr = zeros(size(scanIndex));
        frameStart = P.frameStartEnd(1);
        frameEnd = P.frameStartEnd(2);
        subVideoAddIndex = repmat((0:dvr-1)',1,dvc)+repmat((0:dvc-1)*Vr,dvr,1);
        ffframe = frameEnd-frameStart+1;
        subVideoAddIndex = repmat(subVideoAddIndex,[1 1 ffframe]) + ...
            repmat(reshape((frameStart-1:frameEnd-1)*Vr*Vc,[1 1 ffframe]),dvr,dvc);
        subVideoAddIndex = subVideoAddIndex(:); MM = length(subVideoAddIndex);
        for z = 1:zSearch, % z shift
            tempScanZ = scanIndex(:,:,z);
            if any(tempScanZ(:)),
                videoScan = false(Vr,Vc);
                videoScan(1:ySearch,1:xSearch) = tempScanZ;
                videoNonZeroIndex = find(videoScan); NN = length(videoNonZeroIndex);
                VideoAddIndex = repmat(subVideoAddIndex,1,NN) + ...
                    repmat(videoNonZeroIndex(:)'+(z-1)*Vr*Vc,MM,1);
                videoTensor = video(VideoAddIndex);
                tempU1 = mean(videoTensor);
                temp = zeros(size(tempScanZ));
                temp(tempScanZ) = tempU1;
                u1(:,:,z) = temp;
                temp = zeros(size(tempScanZ));
                temp(tempScanZ) = mean(videoTensor.^2) - tempU1.^2;
                sigma1_sqr(:,:,z) = temp;
            end
        end
        
        Dv = reshape(Dv(:,:,frameStart:frameEnd,:),[],Nk);
        uDv = mean(Dv);
        sigmaDv_sqr = mean(Dv.^2)-uDv.^2;
        
        P.u1 = u1;
        P.sigma1_sqr = sigma1_sqr;
        P.uDv = uDv;
        P.sigmaDv_sqr = sigmaDv_sqr;
    end
else
    P.u1 = [];
    P.sigma1_sqr = [];
    P.uDv = ones(1,Nk);          % which will not be used
    P.sigmaDv_sqr = ones(1,Nk);  % which will not be used
end
