function Seg=segment(X,Window_or_WindowSize,HopSize_or_HopPercent)
% Qingju Liu, University of Surrey
% SEGMENT chops a signal X to overlapping windowed segments
% A= SEGMENT(X,Window,HopSize_or_HopPercent) returns a matrix.
% Its columns are segmented and windowed frames of one dimentional X.
% Window_or_WindowSize is the overlapping window used, default is 256 hamming window.
% it can be the number of the window or a vector.
% HopSize_or_HopPercent is the hopsize or the shift percentage, default value is 0.25.


if nargin<3, HopSize_or_HopPercent=.25;end
if nargin<2, Window_or_WindowSize = 256;end

if length(Window_or_WindowSize)==1,
    Window=hamming(Window_or_WindowSize);
else
    Window=Window_or_WindowSize(:);
end
W = length(Window);

if HopSize_or_HopPercent>1,
    HopSize = round(HopSize_or_HopPercent);
else
    HopSize = round(W.*HopSize_or_HopPercent);
end

L=length(X);
N=fix((L-W)/HopSize +1); %number of segments

Index=(repmat(1:W,N,1)+repmat((0:(N-1))'*HopSize,1,W))';
hw=repmat(Window,1,N);
Seg=X(Index).*hw;
