function VisualSimilarity = visualSimilarityCalculation(dv,videoTensor,similarityType,frame,u1,sigma1_sqr,u_dv,sigma_dv_sqr)
% Qingju Liu, University of Surrey
% VisualSimilarity: Unlike the audioSimilarityCalculation, where only one
% similarity is returned to the result, We calculate each similarity for
% each visual sgement.
%
% dv: visual atom
% videoTensor: each column is associated with one visual segment that
% spanning the same size as dv
% similarityType: to decide what kind of visual similarity is used.
% frame: ditto



switch similarityType,
    case 1 % inner product of dv and the segment
        VisualSimilarity = videoTensor'*dv;
    case 2 % structure similarity index (SSIM)
        sigma12 = videoTensor'*dv/length(dv) - u1*u_dv;
        VisualSimilarity = (2*sigma12 + 0.001)./(sigma1_sqr + sigma_dv_sqr+0.001);
    case 3  % direct correlation
        scanNum = size(videoTensor,2);
        VisualSimilarity = zeros(1,scanNum);
        for i = 1:scanNum,
            temp = corrcoef(dv,videoTensor(:,i));
            VisualSimilarity(i) = temp(1,2);
        end
    case 4  % The visual similarity = exp(-10*difference) * corr(dv,segment)
        scanNum = size(videoTensor,2);
        residual = repmat(dv,[1 scanNum])-videoTensor;
        residual_mean = mean(abs(residual));
        tempError = exp(-10*residual_mean);
        VisualSimilarity = zeros(1,scanNum);
        for i = 1:scanNum,
            temp = corrcoef(dv,videoTensor(:,i));
            VisualSimilarity(i) = temp(1,2);
        end
        VisualSimilarity = tempError(:).*VisualSimilarity(:);
    case 99  % This is specfic for Lip AV data combined with tracking
        VisualSimilarity = 1;
        videoTensor = reshape(videoTensor,[],2,frame);
        for frameIndex = 1:frame,
            % outer lip contour shape similarity
            outerlip1 = dv(1:22,:,frame);
            outerlip = videoTensor(1:22,:,frame);
            temp = outerlip1-outerlip;
            outerDiff =  mean(sum(temp.^2,2));
            %             sigam2_1 = 2*0.15.^2;
            sigam2_1 = 0.045;
            outerSim = exp(-outerDiff/sigam2_1);
            
            % inner lip contour shape similarity
            innerlip1 = dv(23:38,:,frame);
            innerlip = videoTensor(23:38,:,frame);
            temp = innerlip1-innerlip;
            innerDiff =  mean(sum(temp.^2,2));
            %             sigam2_2 = 2*0.1.^2;
            sigam2_2 = 0.02;
            innerSim = exp(-innerDiff/sigam2_2);
            
            % inner lip area shape similarity
            temp = (abs(dv(39,1,frame)) + abs(videoTensor(39,1,frame)))/2;
            if temp<0.2,
                %                 sigam2_3 = 2*0.1.^2;
                sigam2_3 = 0.02;
                w1 = temp*2; w2 = 1-w1;
            else
                sigma = min(0.1+(temp-0.2)/3,0.2);
                sigam2_3 = 2*(sigma).^2;
                w2 = 100^(-temp);
                w1 = 1-w2;
            end
            innerAreaDiff = abs(dv(39,1,frame)-videoTensor(39,1,frame));
            areaSim = exp(-innerAreaDiff^2/sigam2_3);
            
            % overall lip similarity
            visualSim = outerSim * (w1*innerSim + w2*areaSim);
            if visualSim<0.01,
                VisualSimilarity = 0;
                break
            end
            VisualSimilarity = VisualSimilarity*visualSim;
        end
    otherwise  % inner products.
        dv = dv(:);
        VisualSimilarity = dv'*videoTensor;
end

% VisualSimilarity(VisualSimilarity<=10^-6) = 10^-6;
VisualSimilarity(VisualSimilarity<=0) = 0;

end
