function [audio video audio_AV video_AV realscan] = AVMPgenerateAVdata(T,noise_level,seedNumber, figureMode,instan_or_conv)
% Qingju Liu, University of Surrey
% An example of AV data to show the AVMP meathod
if nargin<5,
    instan_or_conv=1; % instan------1, conv, else
end

if nargin<4,
    figureMode = 0;
end
if figureMode,
    seedNumber = 1234; T = 120;
end
Fsa = 16000; %%%% since the audio signal we use is of 16000 sampling rate
Fsv = 30;
%
if nargin<3,
else
    rand('seed',seedNumber);
    randn('seed',seedNumber);
end
if nargin<2,
    noise_level = [];
    % PSNR = 2*noise_level
end
if nargin<1,
    T = 20; % 20 second signal;
end


%*****************************************
% the generating AV functions
video = zeros([5 5 4]);
video(1:2,1:2,1) = 1;
video(2:4,2:3,2) = 1;
video(3:4,2:4,3) = 1;
video(3:4,3:4,4) = 1;
audio_length = round(4*Fsa/Fsv);
load('a.mat');
audio = signalFilt(2000:audio_length+1999);
audio = audio./norm(audio,'fro');
audio = audio(:)';
atom.audio = audio;atom.video = video;
AV{1} = atom;

video = zeros([5 5 4]);
video(2:3,3:5,1:3) = 1;
load('i.mat');
audio = signalFilt(2000:audio_length+1999);
audio = audio./norm(audio,'fro');
audio = audio(:)';
atom.audio = audio;atom.video = video;
AV{2} = atom;


video = zeros([5 5 4]);
video(2:3,2:5,[2,4]) = 1;
load('o.mat');
audio = signalFilt(2000:audio_length+1999);
audio = audio./norm(audio,'fro');
audio = audio(:)';
atom.audio = audio;atom.video = video;
AV{3} = atom;


%%% visual-only atom
video = zeros([5 5 4]);
audio = zeros(1,round(4*Fsa/Fsv));
video(4:5,3:5,1) = 1;
video(3:4,3:4,2) = 1;
video(2:3,3:4,3) = 1;
video(1:2,3:4,4) = 1;
atom.audio = audio;
atom.video = video;
AV{4} = atom;

%%% audio-only atom
video = zeros([5 5 4]);
load('u.mat');
audio = signalFilt(2000:audio_length+1999);
audio = audio./norm(audio,'fro');
audio = audio(:)';
atom.audio = audio;atom.video = video;
AV{5} = atom;
%********************************************************
dar = length(audio);
[dvr dvc frame] = size(video);



%*****************************************
% the generating AV sequence
Nframe = round(T*Fsv);
Naudio = round(T*Fsa);
M = 40; N = 40;
video = zeros(M,N,Nframe);
audio = zeros(1,Naudio);
Ys = M-dvr;
Xs = N-dvc;
Ts = T-frame/Fsv;

realscan = zeros(size(video));

for i = 1:length(AV),
    atom = AV{i};
    audioAtom = atom.audio;
    videoAtom = atom.video;
    atomLocN = round(T*0.8);
    locations = rand(3,atomLocN);
    scales = 0.5 +0.5*rand(3,atomLocN);
    % y position
    locations(1,:) = round(locations(1,:)*Ys)+1;
    % x position
    locations(2,:) = round(locations(2,:)*Xs)+1;
    % time position
    locations(3,:) = locations(3,:)*Ts;
    if figureMode,
        if i==5,
            locations(3,1) = 0.17;
        end
    end
    % video frame position
    TimeVideo = ceil(locations(3,:)*Fsv);
    % audio time position
    TimeAudio = ceil(locations(3,:)*Fsa);
    for j = 1: atomLocN,
        Yloc = locations(1,j);
        Xloc = locations(2,j);
        TVloc = TimeVideo(j);
        TAloc = TimeAudio(j);
        scale = scales(j);
        
        if i<4,
            realscan(Yloc,Xloc,TVloc)=1;
        end
        
        if instan_or_conv,
            disatom = scale*audioAtom;
        else
            b= [randn(1,100).*exp(-(1:100)/20)]; a = 1;
            disatom = filter(b,a,audioAtom);
        end
        
        audio(TAloc:TAloc+dar-1) = audio(TAloc:TAloc+dar-1)+disatom;
        video(Yloc:Yloc+dvr-1,Xloc:Xloc+dvc-1,TVloc:TVloc+frame-1) = ...
            video(Yloc:Yloc+dvr-1,Xloc:Xloc+dvc-1,TVloc:TVloc+frame-1) + videoAtom;
    end
    
    if i==3,
        audio_AV = audio;
        video_AV = video;
    end
end

if ~isempty(noise_level),
    if nargin<3,
        audio = audio+ noise(1,1,audio,noise_level,123456,'R')';
    else
        audio = audio+ noise(1,1,audio,noise_level,seedNumber*100,'R')';
    end
    MSE = 10^(-noise_level*2/10);
    noiseVideo = randn(size(video)); % var = 1;
    
    video = video+sqrt(MSE)*abs(noiseVideo);
end

aaa = max(abs(audio(:)));
audio = audio/aaa;
audio_AV = audio_AV/aaa;

video(video>1)=1;
video_AV(video_AV>1)=1;



if figureMode,
    hFig0 = figure(1);
    tempAudio = 10*log10(abs(stft(audio(1:Fsa),512)));
    colorRange_audio = [min(tempAudio(:)), max(tempAudio(:))];
    colorRange_video = [min(video(:)), max(video(:))];
    tempAudio = [tempAudio(:,[1,1:end,end,end])]; % synchronisation
    displayAVdatall(tempAudio,1-video(:,:,1:Fsv),[],[],[0.35 0.55 0.10],Fsa,Fsv,colorRange_audio,colorRange_video);
    
    %
    set(hFig0, 'Position', [1 1 1600 160]);
    colorRange_audio = [-18 -3];
    for i=1:5,
        hFig = figure(i+1);
        atom = AV{i};
        da = abs(stft(atom.audio,512));
        da = da/(norm(da,'fro')*0.5);
        da(da<0.001*max(da(:))) = 0.001*max(da(:));
        displayAVdata(10*log10(da)/2,1-atom.video,[],[],[0.4 0.4 0.2],Fsa,Fsv,colorRange_audio,colorRange_video);
        set(hFig, 'Position', [100*i-99 1 250 200]);
    end
    
    
end



% function displayAVdata111(audio,video,Col,buf,aabb,Fsa,Fsv)
% % Qingju Liu, University of Surrey
% % To show the 1D/2D audio signal and the synchronous video signal.
% % The video are divided into Col columns.  Default: #frames of the video.
% if numel(audio) == length(audio),   % 1 D
%     audioFlag = 1;
% else                                % 2 D
%     audioFlag = 2;
% end
% video = double(video);
%
% switch audioFlag
%     case 1
%         audio_length = length(audio);
%         audioMax = 1.5*max(abs(audio(:)));
%     case 2
%         [halfNFFT, audio_length] = size(audio);
%         audioMin = min(audio(:));
%         audioMax= max(audio(:));
% end
%
% [Vr Vc Frame] = size(video);
% FsaVsFsv = audio_length/Frame;
%
% if nargin >6 && ~isempty(Fsv),
% else
%     Fsv = 25;
% end
% if nargin >5 && ~isempty(Fsa),
% else
%     Fsa = 16000;
% end
% if nargin >4 && ~isempty(aabb),
% else
%     aabb=[0.2 0.7 0]; % the ratio of audio and video space vertically
% end
% if nargin >3 && ~isempty(buf),
% else
%     buf=1;
% end
% if nargin >2 && ~isempty(Col),
%     Row = ceil(Frame/Col);
% else
%     Col = Frame;
%     Row = 1;
% end
%
% min_video = min(video(:));
% max_video = max(video(:));
% colorScale = [min_video,max_video];
%
% Harray = zeros(1,2*Row);
% for i=1:Row,
%     % The audio signal in the 2i-1 row
%     h = subplot(Row*2,1,(2*i-1));
%     Harray(2*i-1) = h;
%     audio_start = round((i-1)*Col*FsaVsFsv)+1;
%     audio_end = min(round(i*Col*FsaVsFsv),length(audio));
%     switch audioFlag
%         case 1
%             tempAudio = audio(audio_start:audio_end);
%             xaxiss = [1:length(tempAudio)]/length(tempAudio)*Col;
%             plot(xaxiss,tempAudio); xlim([0 Col]);
%             if audioMax>0,
%                 ylim([-audioMax 2*audioMax]);
%             else
%                 ylim([-1 2]);
%             end
%             xlabel('Time (s)'); ylabel('Magnitude');
%
%             t = get(gca,'XTick');
%             realt = t/max(t)*Col/Fsv;
%             realt_string = num2str(realt(:), '%1.2f');
%             set(gca,'XTickLabel',realt_string);
%             set(gca,'Yticklabel',[]);
%             set(gca,'Ytick',[])
%
%             delta = audioMax*0.2;
%             yp = audioMax*1.3;
%             xp = [2.5,6.5];
%             content = 'AV: /a/';
%             plotlinetext(xp,yp,delta,content);
%
%
%             yp = audioMax*0.7;
%             xp = [5,9]+0.1;
%             content = 'Audio only: /u/';
%             plotlinetext(xp,yp,delta,content);
%
%             yp = audioMax*1.3;
%             xp = [10,14]+0.1;
%             content = 'Visual only';
%             plotlinetext(xp,yp,delta,content);
%
%             yp = audioMax*0.7;
%             xp = [11.2,15.2];
%             content = 'AV: /i/';
%             plotlinetext(xp,yp,delta,content);
%
%             yp = audioMax*1.3;
%             xp = [16.5,20.5];
%             content = 'AV: /o/';
%             plotlinetext(xp,yp,delta,content);
%
%             yp = audioMax*1.3;
%             xp = [25,29]+0.1;
%             content = 'Visual only';
%             plotlinetext(xp,yp,delta,content);
%
%     end
%
%     % The video signal in the 2i-1 row
%     h=subplot(Row*2,1,2*i);
%     Harray(2*i) = h;
%     frame_start = ((i-1)*Col+1);
%     frame_end = min(Frame,i*Col);
%     tempVideo = video(:,:,frame_start:frame_end);
%     displayVideo(tempVideo,Col,buf,colorScale)
%     if i==1,
%         set(gca,'YTick',size(tempVideo,1)/2,'YTickLabel','V');
%     end
% end
%
% aa = aabb(1); % ratio of video
% bb = aabb(2); % ratio of audiod
% cc = aabb(3);
% for i=1:Row-1,
%     set(Harray(2*i-1),'position',[0.1 0.05+(Row-i+aa+cc)*0.9/Row 0.85 bb*0.9/Row]);
%     set(Harray(2*i),'position',[0.1 0.05+(Row-i)*0.9/Row 0.85 aa*0.9/Row]);
% end
%
% for i=Row
%     set(Harray(2*i-1),'position',[0.1 0.05+(Row-i+aa+cc)*0.9/Row 0.85*(size(tempVideo,3)/Col) bb*0.9/Row]);
%     set(Harray(2*i),'position',[0.1 0.05+(Row-i)*0.9/Row 0.85*(size(tempVideo,3)/Col) aa*0.9/Row]);
% end




function displayAVdatall(audio,video,Col,buf,aabb,Fsa,Fsv,colorRange_audio,colorRange_video)
% Qingju Liu, University of Surrey
% To show the 1D/2D audio signal and the synchronous video signal.
% The video are divided into Col columns.  Default: #frames of the video.
if numel(audio) == length(audio),   % 1 D
    audioFlag = 1;
else                                % 2 D
    audioFlag = 2;
end
video = double(video);

switch audioFlag
    case 1
        audio_length = length(audio);
        audioMax = 1.5*max(abs(audio(:)));
    case 2
        [halfNFFT, audio_length] = size(audio);
        audioMin = min(audio(:));
        audioMax= max(audio(:));
end

[Vr Vc Frame] = size(video);
FsaVsFsv = audio_length/Frame;

if nargin >6 && ~isempty(Fsv),
else
    Fsv = 25;
end
if nargin >5 && ~isempty(Fsa),
else
    Fsa = 16000;
end
if nargin >4 && ~isempty(aabb),
else
    aabb=[0.2 0.7 0]; % the ratio of audio and video space vertically
end
if nargin >3 && ~isempty(buf),
else
    buf=1;
end
if nargin >2 && ~isempty(Col),
    Row = ceil(Frame/Col);
else
    Col = Frame;
    Row = 1;
end

if nargin >7 && ~isempty(colorRange_audio),
    audioMin = colorRange_audio(1);
    audioMax = colorRange_audio(2);
end

if nargin >8 && ~isempty(colorRange_video),
    colorScale = colorRange_video;
else
    min_video = min(video(:));
    max_video = max(video(:));
    colorScale = [min_video,max_video];
end

Harray = zeros(1,2*Row);
for i=1:Row,
    % The audio signal in the 2i-1 row
    h = subplot(Row*2,1,(2*i-1));
    Harray(2*i-1) = h;
    audio_start = round((i-1)*Col*FsaVsFsv)+1;
    audio_end = min(round(i*Col*FsaVsFsv),length(audio));
    switch audioFlag
        case 1
            tempAudio = audio(audio_start:audio_end);
            xaxiss = [1:length(tempAudio)]/length(tempAudio)*Col;
            plot(xaxiss,tempAudio); xlim([0 Col]);
            if audioMax>0,
                ylim([-audioMax audioMax]);
            else
                ylim([-1 1]);
            end
            xlabel('Time (s)'); ylabel('Magnitude');
            
            t = get(gca,'XTick');
            realt = t/max(t)*Col/Fsv;
            realt_string = num2str(realt(:), '%1.2f');
            set(gca,'XTickLabel',realt_string);
            
            set(gca,'Yticklabel',[]);
        case 2
            tempAudio = audio(:,audio_start:audio_end);
            pcolor(tempAudio); shading interp;caxis([audioMin,audioMax]);
            colorbar;
            freezeColors;
            cbfreeze(colorbar);
            xlabel('Time (s)'); ylabel('Frequency (kHz)');
            
            tTick = [0:0.2:1]*size(tempAudio,2);
            realt = [0:0.2:1]*Col/Fsv;
            realt_string = num2str(realt(:), '%1.2f');
            set(gca,'XTick',tTick,'XTickLabel',realt_string);
            
            %             t = get(gca,'XTick');
            %             realt = t/size(tempAudio,2)*Col/Fsv;
            %             realt_string = num2str(realt(:), '%1.2f');
            %             set(gca,'XTickLabel',realt_string);
            
            fff = [0 0.25 0.5 0.75 1];
            real_values = round(fff*(size(tempAudio,1)-1));
            realf_string = num2str(fff(:)*Fsa/2000, '%1.0f');
            set(gca,'YTick',real_values,'YTickLabel',realf_string);
            
            delta = size(tempAudio,1)/15;
            yp = size(tempAudio,1)*0.8;
            xp = [2.2,6.2]/60*size(tempAudio,1);
            content = 'AV: /a/';
            plotlinetext(xp,yp,delta,content);
            
            yp = size(tempAudio,1)*0.6;
            xp = [5,9]/60*size(tempAudio,1);
            content = 'AV: /u/';
            plotlinetext(xp,yp,delta,content);
            
            
            yp = size(tempAudio,1)*0.8;
            xp = [10,14]/60*size(tempAudio,1);
            content = 'Visual only';
            plotlinetext(xp,yp,delta,content);
            
            yp = size(tempAudio,1)*0.6;
            xp = [10.5,14.5]/60*size(tempAudio,1);
            content = 'AV: /i/';
            plotlinetext(xp,yp,delta,content);
            
            yp = size(tempAudio,1)*0.8;
            xp = [15.6,19.6]/60*size(tempAudio,1);
            content = 'AV: /o/';
            plotlinetext(xp,yp,delta,content);
            
            yp = size(tempAudio,1)*0.1;
            xp = [24,28]/60*size(tempAudio,1);
            content = 'Visual only';
            plotlinetext(xp,yp,delta,content);
            
    end
    
    % The video signal in the 2i-1 row
    h=subplot(Row*2,1,2*i);
    Harray(2*i) = h;
    frame_start = ((i-1)*Col+1);
    frame_end = min(Frame,i*Col);
    tempVideo = video(:,:,frame_start:frame_end);
    displayVideo(tempVideo,Col,buf,colorScale);
    if i==1,
        set(gca,'YTick',size(tempVideo,1)/2,'YTickLabel','V');
    end
end

aa = aabb(1); % ratio of video
bb = aabb(2); % ratio of audiod
cc = aabb(3);
for i=1:Row-1,
    set(Harray(2*i-1),'position',[0.1 0.05+(Row-i+aa+cc)*0.9/Row 0.85 bb*0.9/Row]);
    set(Harray(2*i),'position',[0.1 0.05+(Row-i)*0.9/Row 0.85 aa*0.9/Row]);
end

for i=Row
    set(Harray(2*i-1),'position',[0.1 0.05+(Row-i+aa+cc)*0.9/Row 0.85*(size(tempVideo,3)/Col) bb*0.9/Row]);
    set(Harray(2*i),'position',[0.1 0.05+(Row-i)*0.9/Row 0.85*(size(tempVideo,3)/Col) aa*0.9/Row]);
end


function plotlinetext(xp,yp,delta,content)
X = [xp(1),xp(1)];Y = [yp-delta,yp+delta];line(X,Y,'Color','k');
X = [xp(2),xp(2)];Y = [yp-delta,yp+delta];line(X,Y,'Color','k');
X = xp;Y = [yp,yp];line(X,Y,'Color','k');
text(mean(xp),yp+2*delta,content,'HorizontalAlignment','center','Color','k');