function displayAVdata(audio,video,Col,buf,aabb,Fsa,Fsv,colorRange_audio,colorRange_video)
% Qingju Liu, University of Surrey
% To show the 1D/2D audio signal and the synchronous video signal.
% The video are divided into Col columns.  Default: #frames of the video.
if numel(audio) == length(audio),   % 1 D
    audioFlag = 1;
else                                % 2 D
    audioFlag = 2;
end
video = double(video);

switch audioFlag
    case 1
        audio_length = length(audio);
        audioMax = 1.5*max(abs(audio(:)));
    case 2
        [halfNFFT, audio_length] = size(audio);
        audioMin = min(audio(:));
        audioMax= max(audio(:));
end

[Vr Vc Frame] = size(video);
FsaVsFsv = audio_length/Frame;

if nargin >6 && ~isempty(Fsv),
else
    Fsv = 25;
end
if nargin >5 && ~isempty(Fsa),
else
    Fsa = 16000;
end
if nargin >4 && ~isempty(aabb),
else
    aabb=[0.2 0.7 0]; % the ratio of audio and video space vertically
end
if nargin >3 && ~isempty(buf),
else
    buf=1;
end
if nargin >2 && ~isempty(Col),
    Row = ceil(Frame/Col);
else
    Col = Frame;
    Row = 1;
end

if nargin >7 && ~isempty(colorRange_audio),
    audioMin = colorRange_audio(1);
    audioMax = colorRange_audio(2);
end

if nargin >8 && ~isempty(colorRange_video),
    colorScale = colorRange_video;
else
    min_video = min(video(:));
    max_video = max(video(:));
    colorScale = [min_video,max_video];
end

Harray = zeros(1,2*Row);
for i=1:Row,
    % The audio signal in the 2i-1 row
    h = subplot(Row*2,1,(2*i-1));
    Harray(2*i-1) = h;
    audio_start = round((i-1)*Col*FsaVsFsv)+1;
    audio_end = min(round(i*Col*FsaVsFsv),length(audio));
    switch audioFlag
        case 1
            tempAudio = audio(audio_start:audio_end);
            xaxiss = [1:length(tempAudio)]/length(tempAudio)*Col;
            plot(xaxiss,tempAudio); xlim([0 Col]);
            if audioMax>0,
                ylim([-audioMax audioMax]);
            else
                ylim([-1 1]);
            end
            xlabel('Time (s)'); ylabel('Magnitude');
            
            t = get(gca,'XTick');
            realt = t/max(t)*Col/Fsv;
            realt_string = num2str(realt(:), '%1.2f');
            set(gca,'XTickLabel',realt_string);
            
            set(gca,'Yticklabel',[]);
        case 2
            tempAudio = audio(:,audio_start:audio_end);
            pcolor(tempAudio); shading interp;caxis([audioMin,audioMax]);
            colormap('default')
            colorbar;
            freezeColors;
            cbfreeze(colorbar);
            xlabel('Time (s)'); ylabel('Frequency (kHz)');
            
            t = get(gca,'XTick');
            realt = t/size(tempAudio,2)*Col/Fsv;
            realt_string = num2str(realt(:), '%1.2f');
            set(gca,'XTickLabel',realt_string);
            
            
            fff = [0 0.25 0.5 0.75 1];
            real_values = round(fff*(size(tempAudio,1)-1));
            realf_string = num2str(fff(:)*Fsa/2000, '%1.0f');
            set(gca,'YTick',real_values,'YTickLabel',realf_string);
    end
    
    % The video signal in the 2i-1 row
    h=subplot(Row*2,1,2*i);
    Harray(2*i) = h;
    frame_start = ((i-1)*Col+1);
    frame_end = min(Frame,i*Col);
    tempVideo = video(:,:,frame_start:frame_end);
    displayVideo(tempVideo,Col,buf,colorScale);
    if i==1,
        set(gca,'YTick',size(tempVideo,1)/2,'YTickLabel','V');
    end
end

aa = aabb(1); % ratio of video
bb = aabb(2); % ratio of audiod
cc = aabb(3);
for i=1:Row-1,
    set(Harray(2*i-1),'position',[0.1 0.05+(Row-i+aa+cc)*0.9/Row 0.85 bb*0.9/Row]);
    set(Harray(2*i),'position',[0.1 0.05+(Row-i)*0.9/Row 0.85 aa*0.9/Row]);
end

for i=Row
    set(Harray(2*i-1),'position',[0.1 0.05+(Row-i+aa+cc)*0.9/Row 0.85*(size(tempVideo,3)/Col) bb*0.9/Row]);
    set(Harray(2*i),'position',[0.1 0.05+(Row-i)*0.9/Row 0.85*(size(tempVideo,3)/Col) aa*0.9/Row]);
end
